package it.istat.relais.batch;

import it.istat.relais.constants.ReleaseName;
import it.istat.relais.datasetInput.ReadingFile;
import it.istat.relais.datasetInput.Record;
import it.istat.relais.datasetInput.RecordReader;
import it.istat.relais.export.ExportToFile;
import it.istat.relais.linkageOtoO.LinkageOtoO;
import it.istat.relais.linkageOtoO.LinkageOtoOBlocking;
import it.istat.relais.linkageOtoO.ReductionByDB;
import java.sql.Connection;
import it.istat.relais.project.DBConfiguration;
import it.istat.relais.project.VariablesSetting;
import it.istat.relais.searchSpaceCreation.CrossProduct;
import it.istat.relais.searchSpaceCreation.Blocking;
import it.istat.relais.searchSpaceCreation.SortedNeighborhood;
import it.istat.relais.result.ResultTable;
import it.istat.relais.matchingVariables.MatchingVariables;
import it.istat.relais.method.fs.FellegiSunterBlocking;
import it.istat.relais.method.fs.FellegiSunterMethod;
import it.istat.relais.metrics.dataStructure.*;
import it.istat.relais.constants.TablesName;
import it.istat.relais.decision.contingencyTable.ContingencyTable;
import it.istat.relais.deterministic.MUTableDeterministic;
import it.istat.relais.deterministic.ReducedWeight;
import it.istat.relais.deterministic.RuleElement;
import it.istat.relais.searchSpaceCreation.NestedBlocking;
import it.istat.relais.searchSpaceCreation.Simhash;
import java.util.StringTokenizer;


public class RelaisBatch {

    private final String title="Relais batch "+ReleaseName.release()+" (beta)";
    private String inputParam;
    private String dsaFile;
    private String dsaSep;
    private String dsbFile;
    private String dsbSep;
    private String dedup;
    private String method;
    private String blockVar;
    private String sortVar;
    private int windowDim;
    private String model;
    private String rule;
    private MetricMatchingVariableVector mVariables;
    private String lkOtoO;
    private double tU;
    private double tM;
    private String outputFolder;
    private String sep;
    private int[] vSHPar;
    private String residuals;
    
    public RelaisBatch(String paramFile) {
        this.inputParam = paramFile;
        this.sep = "=";
        this.dsaFile = "";
        this.dsaSep = "";
        this.dsbFile = "";
        this.dsbSep = "";
        this.dedup = "N";
        this.method = "CrossProduct"; 
        this.blockVar = "";
        this.sortVar = "";
        this.windowDim = 0;
        this.model = ""; 
        this.rule = "";
        this.mVariables = new MetricMatchingVariableVector();
        this.lkOtoO = "";
        this.tU = 0.90;
        this.tM = 0.95;
        this.outputFolder = "";
        /*0->inputed,1->nGramSize,2->hdThr,3->rounds,4->weigths*/
        this.vSHPar = new int[]{0,2,45,4,0};
        this.residuals="No";

    }

    public void relaisBatch() throws Exception{
        if (inputParam.equalsIgnoreCase("-help"))
            relaisBatchHelp();
        else
            relaisBatchElab();
    }
    public void relaisBatchElab() throws Exception{
        RecordReader readerIF;
        Record row;
        String rs;
        Record vmr;
        String sep2 = ",";
        String nameMethod;
        CrossProduct cp;
        Blocking b;
        SortedNeighborhood snm;
        NestedBlocking nbl;
        Simhash sh;
        MetricMatchingVariable mmv;
        String strMVar="";
        String strMMet="";
        String strMThr="";
        String appoTU="";
        String appoTM="";
        String appoWS="";
        String strSHParam="";
        int nMVar=0;
        ExportToFile exf;
        System.out.println(title);
        
       try {
            readerIF = new RecordReader(this.inputParam, this.sep);
            while ((rs = readerIF.read()) != null) {
                //System.out.println(rs);
                row = new Record(rs, this.sep);

                if (row.get(0).toString().trim().equalsIgnoreCase("dsaFile")) {
                    this.dsaFile = row.get(1).toString().trim();
                } else if (row.get(0).toString().trim().equalsIgnoreCase("dsbFile")) {
                    this.dsbFile = row.get(1).toString().trim();
                } else if (row.get(0).toString().trim().equalsIgnoreCase("dsaSep")) {
                    this.dsaSep = row.get(1).toString().trim();
                } else if (row.get(0).toString().trim().equalsIgnoreCase("dsbSep")) {
                    this.dsbSep = row.get(1).toString().trim();
                } else if (row.get(0).toString().trim().equalsIgnoreCase("dedup")) {
                    this.dedup = row.get(1).toString().trim().toUpperCase();
                } else if (row.get(0).toString().trim().equalsIgnoreCase("Method")) {
                    this.method = row.get(1).toString().trim();
                } else if (row.get(0).toString().trim().equalsIgnoreCase("BKey")) {
                    this.blockVar = row.get(1).toString().trim();
                } else if (row.get(0).toString().trim().equalsIgnoreCase("SKey")) {
                    this.sortVar = row.get(1).toString().trim();
                } else if (row.get(0).toString().trim().equalsIgnoreCase("WSize")) {
                    appoWS = row.get(1).toString().trim();
                } else if (row.get(0).toString().trim().equalsIgnoreCase("SHParam")) {
                    strSHParam = row.get(1).toString().trim();
                } else if (row.get(0).toString().trim().equalsIgnoreCase("Model")) {
                    this.model = row.get(1).toString().trim();
                } else if (row.get(0).toString().trim().equalsIgnoreCase("Rule")) {
                    this.rule = row.get(1).toString();
                    int appo=2;
                    while (appo<row.size()) {
                        this.rule = this.rule + "=" + row.get(appo).toString();
                        appo++;
                    } 
                } else if (row.get(0).toString().trim().equalsIgnoreCase("MVar")) {
                    strMVar = row.get(1).toString();
                } else if (row.get(0).toString().trim().equalsIgnoreCase("MMet")) {
                    strMMet = row.get(1).toString();
                } else if (row.get(0).toString().trim().equalsIgnoreCase("MThr")) {
                    strMThr = row.get(1).toString();
                } else if (row.get(0).toString().trim().equalsIgnoreCase("1to1")) {
                    this.lkOtoO = row.get(1).toString().trim();
                } else if (row.get(0).toString().trim().equalsIgnoreCase("TU")) {
                    appoTU=row.get(1).toString().trim();
                } else if (row.get(0).toString().trim().equalsIgnoreCase("TM")) {
                    appoTM = row.get(1).toString().trim();
                } else if (row.get(0).toString().trim().equalsIgnoreCase("Folder")){
                    this.outputFolder = row.get(1).toString().trim();
                } else if (row.get(0).toString().trim().equalsIgnoreCase("Residuals")){
                    this.residuals = row.get(1).toString().trim();
                } 
            }
            readerIF.close();
            
            /*verify parameters*/
            /*Read data set*/
            if (dsaSep.equals("")) {
                System.out.println("ERROR: parameter dsaSep - missing value!");
                return;
            }
            if (dsaFile.equals("")) {
                System.out.println("ERROR: parameter dsaFile - missing value!");
                return;
            }
            if (!dedup.startsWith("Y") && dsbSep.equals("")) {
                System.out.println("ERROR: parameter dsbSep - missing value!");
                return;
            }
            if (!dedup.startsWith("Y") && dsbFile.equals("")) {
                System.out.println("ERROR: parameter dsbFile - missing value!");
                return;
            }
            /* search space */
            if (method.equals("")) method="CrossProduct";
            if (method.toUpperCase().startsWith("CRO")) method="CrossProduct";
            if (method.toUpperCase().startsWith("BLO")) 
            {
                if (method.toUpperCase().contains("SIMH"))
                    method="Blocked SimHash";
                else if (method.toUpperCase().contains("UNI"))
                    method="Blocking Union";
                else if (method.length()<10)
                  method="Blocking";
            }
            if (method.toUpperCase().startsWith("SIMH"))  method="SimHash";
            if (method.toUpperCase().startsWith("NES")) method="NestedBlocking";
            if (method.toUpperCase().startsWith("SOR")) method="SortedNeighborhood";
            
            if (!method.equalsIgnoreCase("CrossProduct") && 
                !method.equalsIgnoreCase("Blocking") &&
                !method.equalsIgnoreCase("NestedBlocking") &&
                !method.equalsIgnoreCase("SortedNeighborhood") &&
                !method.equalsIgnoreCase("Blocking Union") &&
                !method.equalsIgnoreCase("SimHash") &&
                !method.equalsIgnoreCase("Blocked SimHash")) {
                System.out.println("ERROR: parameter method - the value is not correct!");
                return;
            }            
            System.out.println("Search space creation method: "+this.method); 
            
            if (appoWS.equals("")) {
                windowDim=0;
            }
            else try {
                windowDim = Integer.parseInt(appoWS);
            } catch (Exception ex) {
                System.out.println("ERROR: perameter WSize - '"+appoWS+"' is not numeric!");
                return;
            }
            
            if (method.contains("Blocking") && blockVar.equals("")) {
                System.out.println("ERROR: parameter BKey - missing value!");
                return;
            }
            if (method.contains("Blocked") && blockVar.equals("")) {
                System.out.println("ERROR: parameter BKey - missing value!");
                return;
            }
            if (method.equalsIgnoreCase("SortedNeighborhood") && sortVar.equals("")) {
                System.out.println("ERROR: parameter SKey - missing value!");
                return;
            }
            if (method.equalsIgnoreCase("SortedNeighborhood") && windowDim==0) {
                System.out.println("ERROR: parameter WSize - missing value!");
                return;
            }
            if (method.equalsIgnoreCase("NestedBlocking") && sortVar.equals("")) {
                System.out.println("ERROR: parameter SKey - missing value!");
                return;
            }
            if (method.equalsIgnoreCase("NestedBlocking") && windowDim==0) {
                System.out.println("ERROR: parameter WSize - missing value!");
                return;
            }
            if (method.contains("SimHash") && sortVar.equals("")) {
                System.out.println("ERROR: parameter SKey - missing value!");
                return;
            }
            
            /* SimHash Parameters (optional)*/
            if (!strSHParam.equals("")) {
                int nPar=0;
                vSHPar[0] = 1;
                String SParAppo;
                StringTokenizer shpar = new StringTokenizer(strSHParam,sep2);
                /*0->inputed,1->nGramSizehdTr,2->hdThr,3->rounds,4->weigths*/
                while (shpar.hasMoreTokens()) {
                   nPar++;
                   SParAppo=shpar.nextToken();
                   try {
                      vSHPar[nPar] = Integer.parseInt(SParAppo);
                   } catch (Exception ex) {
                      System.out.println("ERROR: perameter SHParam - '"+SParAppo+"' is not integer!");
                      return;
                   }
                  
                }
                
                if (vSHPar[1]<2 || vSHPar[1]>4) {
                    System.out.println("ERROR: perameter SHParam#1 is not correct (not between 2 and 4)!");
                    return;
                }
                if (vSHPar[3]!=2 && vSHPar[3]!=4 && vSHPar[3]!=8 && vSHPar[3]!=16 && vSHPar[3]!=32) {
                    System.out.println("ERROR: perameter SHParam#3 is not correct (admitted values are 2,4,8,16 and 32)!");
                    return;
                }
                if (vSHPar[4]!=0 && vSHPar[4]!=1) {
                    System.out.println("ERROR: perameter SHParam#4 is not correct (admitted values 0 and 1)!");
                    return;
                }
            }
            
            
            /* matching variables */
            if (strMVar.equals("")) {
                System.out.println("ERROR: parameter MVar - missing value!");
                return;
            }
            System.out.println("Matching variables: ");
            StringTokenizer variables = new StringTokenizer(strMVar,sep2);
            StringTokenizer metrics = new StringTokenizer(strMMet,sep2);
            StringTokenizer thresholds = new StringTokenizer(strMThr,sep2);
            String newmet,newvar,appothr;
            Double newthr;
            while (variables.hasMoreTokens()) {
               nMVar++;
               newvar=variables.nextToken();
               newmet="";
               newthr=1.0;
               if (metrics.hasMoreTokens()) newmet=metrics.nextToken();
               if (newmet.equals("")) newmet="Equality";
               if (!newmet.equals("Equality") && !newmet.equals("Jaro") && 
                   !newmet.equals("Dice") && !newmet.equals("JaroWinkler") && 
                   !newmet.equals("Levenshtein") && !newmet.equals("3Grams") &&
                   !newmet.equals("Soundex") && !newmet.equals("NumericComparison")) {
                    System.out.println("ERROR: parameter MMet - value '"+newmet+"' is not correct!");
                    return;
               }
               if (thresholds.hasMoreTokens()) {
                   appothr=thresholds.nextToken();
                   try {
                      newthr = Double.parseDouble(appothr);
                   } catch (Exception ex) {
                      System.out.println("ERROR: perameter MThr - '"+appothr+"' is not numeric!");
                      return;
                   }
               }
               mmv = new MetricMatchingVariable(newvar, newmet, newthr,1);
               mVariables.addElement(mmv);
               System.out.println("   "+nMVar+" - "+newvar+" ("+newmet+","+newthr+")");
            }
            
            /* model */
            if (model.equals("")) {
                System.out.println("ERROR: parameter Model - missing value!");
                return;
            }
            if (model.toUpperCase().startsWith("FEL") ||
                model.toUpperCase().startsWith("FS") ||
                model.toUpperCase().startsWith("PROB")) model="Fellegi-Sunter";
            if (model.toUpperCase().startsWith("DET") ||
                model.toUpperCase().startsWith("EXA")) model="Exact";
            if (model.toUpperCase().startsWith("RUL")) model="RuleBased";
            if (!model.equals("Fellegi-Sunter") &&
                !model.equals("Exact") &&
                !model.equals("RuleBased")) {
                      System.out.println("ERROR: perameter Model - value '"+model+"' is not correct!");
                      return;
                   }
            System.out.println("Decision model: "+this.model);
            
            if (model.equals("Fellegi-Sunter")) {
                if (nMVar<3) {
                    System.out.println("ERROR: for "+model+" model at least 3 matching variables are required!");
                    return;
                }
            }
            if (model.equals("Exact")) {
                model="Deterministic";
                method="Exact";
            }
            if (model.equals("RuleBased")) {
                model="Deterministic";
                if (rule.equals("")) {
                    System.out.println("ERROR: perameter Rule - missing value!");
                    return;
                }
            }
                
            /* 1:1 reduction */
            if (lkOtoO.toUpperCase().startsWith("N") ||
                lkOtoO.equals("")) lkOtoO="No";
            if (lkOtoO.toUpperCase().startsWith("OP")) lkOtoO="Optimal";
            if (lkOtoO.toUpperCase().startsWith("GR")) lkOtoO="Greedy";
            if (!lkOtoO.equals("No") && !lkOtoO.equals("Optimal") &&
                !lkOtoO.equals("Greedy")) {
                System.out.println("ERROR: perameter 1to1 - value '"+lkOtoO+"' is not correct!");
                return;
            }
            System.out.println("1:1 Reduction method: "+this.lkOtoO);
        
            /* Thresholds and Output folder */
            if (model.equals("Fellegi-Sunter")) {
                if (appoTU.equals("")) {
                    System.out.println("ERROR: parameter TU - missing value!");
                    return;
                }
                try {
                   this.tU = Double.parseDouble(appoTU);
                } catch (Exception ex) {
                   System.out.println("ERROR: perameter TU - '"+appoTU+"' is not numeric!");
                   return;
                }
                if (appoTM.equals("")) {
                    System.out.println("ERROR: parameter TM - missing value!");
                    return;
                }
                try {
                   this.tM = Double.parseDouble(appoTM);
                } catch (Exception ex) {
                   System.out.println("ERROR: perameter TM - '"+appoTM+"' is not numeric!");
                   return;
                }
                System.out.println("Unmatch threshold: "+this.tU);
                System.out.println("Match threshold:   "+this.tM);
            }
            
            if (outputFolder.equals("")) {
                System.out.println("ERROR: parameter Folder - missing value!");
                return;
            }
            System.out.println("Output folder: "+outputFolder);
            
            if (residuals.equalsIgnoreCase("Yes") || residuals.equalsIgnoreCase("Y") || residuals.equalsIgnoreCase("S") || residuals.equalsIgnoreCase("Si")) 
                 residuals="Yes";
            else residuals="No";
            
            System.out.println("Save residuals: "+residuals);
            
            //Connection to database
            Connection conn = DBConfiguration.getConnection();
            VariablesSetting vs = new VariablesSetting(conn);
            try {
                Class.forName("com.mysql.jdbc.Driver");
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            }

            try {
                conn = DBConfiguration.getConnection();
                vs.InitialSettingVariables();
            } catch (Exception ex) {
                System.out.println(ex.getMessage());
            }
            //End connection to database

            //Input of dataset
            System.out.println("Input of dataset");
            
            vs.UpdateSeparator(dsaSep);
            ReadingFile ri = new ReadingFile(conn);
            if (dedup.startsWith("Y")) {
                vs.UpdateProjectType("Deduplication");
                ri.Reading(dsaFile, dsaSep, dsaFile, dsaSep);
            } else {
                vs.UpdateProjectType("Linkage");
                ri.Reading(this.dsaFile, this.dsaSep, this.dsbFile, this.dsbSep);
            }
            //End input of dataset
           
            //Search Space Creation
            System.out.println("Search Space Creation");
            String blocksortVar="";
            nameMethod="";
            
            if (this.method.equalsIgnoreCase("CrossProduct")) {
                cp = new CrossProduct(conn);
                nameMethod = cp.CreateCrossProduct();
            } else if (this.method.equalsIgnoreCase("Blocking")) {
                b = new Blocking(conn);
                nameMethod = b.CreateBlocks(blockVar);
                blocksortVar = blockVar;
            } else if (this.method.equalsIgnoreCase("SortedNeighborhood")) {
                snm = new SortedNeighborhood(conn);
                nameMethod = snm.CreateSNM(sortVar,windowDim);
                blocksortVar = sortVar;
            } else if (this.method.equalsIgnoreCase("NestedBlocking")) {
                nbl = new NestedBlocking(conn);
                nameMethod = nbl.CreateSSR(blockVar, sortVar, windowDim);
                blocksortVar = blockVar+","+sortVar;
            } else if (this.method.equalsIgnoreCase("Blocking Union")) {
                b = new Blocking(conn);
                nameMethod = b.CreateBlocks(blockVar);
                nameMethod = "Blocking Union";
            } else if (this.method.equals("SimHash")) {
                /*0->inputed,1->nGramSize,2->hdThr,3->rounds,4->weigths*/
                System.out.println("SH params : hdthres="+vSHPar[2]+", rotations="+vSHPar[3]+", grams size="+vSHPar[1]+", use of weight="+(vSHPar[4]==1));
                sh = new Simhash(conn,vSHPar[2],vSHPar[3],vSHPar[1],(vSHPar[4]==1));
                nameMethod = sh.CreateSSR(sortVar);
            } else if (this.method.equals("Blocked SimHash")) {
                System.out.println("SH params : hdthres="+vSHPar[2]+", rotations="+vSHPar[3]+", grams size="+vSHPar[1]+", use of weight="+(vSHPar[4]==1));
                sh = new Simhash(conn,vSHPar[2],vSHPar[3],vSHPar[1],(vSHPar[4]==1));
                nameMethod = sh.CreateSSRBlock(sortVar,blockVar);
            }
        //End Search Space Creation
            
            //vs Setting
              vs.UpdateMethod(nameMethod);
              vs.UpdateBlockVar(blocksortVar);
              vs.UpdateModel(this.model);
              vs.UpdateRule(this.rule);
              vs.UpdateThresholdU(this.tU);
              vs.UpdateThresholdM(this.tM);
            //End vs Setting
            //Rule based
              
           if(this.model.equals("Deterministic")&& (!this.method.equals("Exact"))){
              System.out.println("Model: Deterministic Rule Based - Write rule");
               WriteRuleBatch(rule, conn); 
            }
            //End Rule based
            //Matching Variables
           if(!this.model.equals("Deterministic")){
            System.out.println("Matching Variables");
              MatchingVariables mv;
            mv = new MatchingVariables(conn, this.mVariables);
            mv.CreateMatchingVariablesTable();
            mv.UpdateMatchingVariableTable(this.mVariables);
           }
            //End Matching Variables
        //ContingencyTableOLd
            ContingencyTable ctg;
            ctg = new ContingencyTable(conn);
            if(this.model.equals("Fellegi-Sunter")){
                if (this.method.endsWith("Blocking")) {
                System.out.println("Contingency table - Blocking");
                    ctg.CreateContingencyTableBlocking();
                } 
                else {
                System.out.println("Contingency table - CP or SNM");
                    ctg.CreateContingencyTable();            
                }
            }
            if (this.model.equals("Deterministic")) {
                vs.UpdateThresholdU(0.5);
                vs.UpdateThresholdM(1);
                if (method.equals("Exact")){
                    System.out.println("Contingency table - Exact");
                    ResultTable rt = new ResultTable(conn);
                    rt.CreateTableMatch("key_dsa");
                    rt.CreateTableUnmatchDSA();
                    rt.CreateTableUnmatchDSB();
                } else {
                  System.out.println("Contingency table - CP or SNM - Rule Based");
                  ctg.CreateContingencyTable();     
                }  
                }
            
            //End ContingencyTableOLd
            
            //EM Reduction
            if((this.model.equals("Fellegi-Sunter"))){
            try {
              if (method.endsWith("Blocking")) {
                System.out.println("FellegiSunter - Blocking");
                FellegiSunterBlocking fsb = new FellegiSunterBlocking(conn);
                fsb.FellegiSunter();
              } else {
                System.out.println("FellegiSunter - CP or SNM");
                FellegiSunterMethod fs = new FellegiSunterMethod(conn);
                fs.FellegiSunter();
              }
              
             } catch (Exception ex) {
                exf = new ExportToFile(this.outputFolder, conn); 
                exf.expTable("marginals.txt", TablesName.nameMUVariable());
                System.out.println("Saved marginal probabilities in output folder");

                return;
             }
            }
            //End EM Reduction
            
            //Reduction 1:1
            if(this.model.equals("Deterministic")&& (!this.method.equals("Exact"))){
              /*weights for subrules*/
               ReducedWeight rw = new ReducedWeight(conn);
               double[] wgs = rw.getWeights(); 
               rw.updateWeights(wgs);
            }
            
            if(!this.lkOtoO.equals("No")){
            if (this.lkOtoO.equals("Optimal") && method.equals("Blocking")) {
                System.out.println("1to1 - LP -Blocking");
                
                LinkageOtoOBlocking lob = new LinkageOtoOBlocking(conn);
                lob.ReduceOtoO();
            } else if (this.lkOtoO.equals("Optimal")) {
                System.out.println("1to1 - LP - CP or SNM");
                
                LinkageOtoO loo = new LinkageOtoO(conn);
                loo.ReduceOtoO();  
            } else { 
                ReductionByDB rdb = new ReductionByDB(conn);
                
                if (method.endsWith("Blocking")) {
                    System.out.println("1to1 - Greedy - Blocking");
                    rdb.ReduceOtoOBlock();
                }
                else {
                    System.out.println("1to1 - Greedy - CP or SNM");
                    rdb.ReduceOtoO();
                }
                System.out.println("Reduction 1to1 completed");
            }
            }
            //End reduction 1:1
            
            //Write result table
            
            ResultTable rt = new ResultTable(conn);
           
            if (!this.lkOtoO.equals("No")) {
              //Reduction 1:1 
                System.out.println("Write result Table - reduction 1to1");
                    
                    rt.CreateTableMatchOtoO();
                    rt.CreateTablePossiblematchOtoO();
                    //rt.CreateTableUnmatchOtoO();
                    if (residuals.equals("Yes")) {
                      rt.CreateTableUnmatchDSA();
                      rt.CreateTableUnmatchDSB();
                    }
            } else if(!(this.method.equals("Exact"))) { /*no reduction && no deterministic Exact*/ 
                System.out.println("Write result Table - not reduction - not exact");
                    rt.CreateTableMatch("key_dsa");
                    rt.CreateTablePossiblematch("key_dsa");
                    //rt.CreateTableUnmatch("key_dsa");
                    if (residuals.equals("Yes")) {
                       rt.CreateTableUnmatchDSA();
                       rt.CreateTableUnmatchDSB();
                    }
            }
             //End write result table
            
            //Write result File
            System.out.println("Write result Files");
                  
            exf = new ExportToFile(this.outputFolder, conn); 
                //Match
            exf.expTableMPU("match.txt", TablesName.nameMatch());
            //Possible Match
            if(!(this.model.equals("Deterministic Exact"))){
            exf.expTableMPU("possibleMatch.txt", TablesName.namePossibleMatch());
                //Unmatch
            //exf.expTableMPU("unmatch.txt", TablesName.nameUnMatch());
                //Residual
            }
            if (residuals.equals("Yes")) {
               exf.expTableDS("residualDSA.txt",TablesName.nameResidualDsa());
               exf.expTableDS("residualDSB.txt",TablesName.nameResidualDsb());
            }
             //if (!method.endsWith("Blocking")) exf.expTable("contingency.txt", TablesName.nameContingency());
           
             //End Write result File
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw e;
        }

    }

    private void WriteRuleBatch(String rule, Connection conn) {
     Connection connection;
     RuleElement[] elementsRule;
     int lastElement, indexMV;
     MetricMatchingVariableVector mmvv;
     String[] token;
     String matchingVariable = null;
     String comparisonFunction = null;
     double threshold = 1.0;
     String rulestd="";
     
     for(int k= 0; k<rule.length();k++) {
         if (rule.charAt(k)=='(' || rule.charAt(k)=='[' || rule.charAt(k)==')' ||
              rule.charAt(k)=='>') {
             if (k>0 && rule.charAt(k-1)!=' ' && rule.charAt(k-1)!=']') rulestd+=' ';
         } 
         rulestd+=rule.charAt(k);
         if (rule.charAt(k)=='(' || rule.charAt(k)==')' || rule.charAt(k)==']' ||
              rule.charAt(k)=='=') {
             if ((k+1)<rule.length() && rule.charAt(k+1)!=' ') rulestd+=' ';
         }
     }
                
     System.out.println("Reading rule: "+rulestd);
     connection=conn;
     mmvv = new MetricMatchingVariableVector();
     elementsRule = new RuleElement[200];
     lastElement = 0;
     indexMV = 0;
     elementsRule[lastElement]=new RuleElement(lastElement,"",1,1,'(',' ',false);
     
     token = new String[500];
      int i=0;  
     token = rulestd.split(" ");
     while(i < token.length){
         System.out.print(token[i]+"+");
         if(token[i].equals("(")){
             if(i>0 && token[i-1].equalsIgnoreCase("OR")){
                elementsRule[lastElement+1]=new RuleElement(lastElement+1," (",
                                  elementsRule[lastElement].GetLevel()+1,
                                  -1,'(',elementsRule[lastElement].GetOperator(),elementsRule[lastElement].GetNot());
                lastElement++;
           }else {
            elementsRule[lastElement]=new RuleElement(lastElement,"",1,1,'(',' ',false);
            elementsRule[lastElement+1]=new RuleElement(lastElement+1," (",
                                  elementsRule[lastElement].GetLevel()+1,
                                  -1,'(',elementsRule[lastElement].GetOperator(),elementsRule[lastElement].GetNot());
            lastElement++;
           }
        }else if(token[i].equals("Equality")||token[i].equals("NumericComparison")||token[i].equals("3Grams")||token[i].equals("Dice")||token[i].equals("Jaro")||token[i].equals("JaroWinkler")||token[i].equals("Levenshtein")||token[i].equals("Soundex")){
            comparisonFunction = token[i];
        }else if(token[i].contains("[")){
            matchingVariable ="";
            for(int k= 1; k<token[i].length()-1;k++)
               matchingVariable += token[i].charAt(k);
        }else if(token[i].equals(">=")){
            threshold = Double.parseDouble(token[i+1]);
        }else if(token[i].equalsIgnoreCase("AND")){
        //Cose da fare quando finisce una regola (era add condition)
            int currMV;
            if (elementsRule[lastElement].GetLastdigit() != '(') {
                elementsRule[lastElement+1]=new RuleElement(lastElement+1," And",
                                      elementsRule[lastElement].GetLevel(),
                                      -1,'O','A',false);
            lastElement++;
            }  
       
            char oper = elementsRule[lastElement].GetOperator();
            boolean not = elementsRule[lastElement].GetNot();
      
            if (elementsRule[lastElement].GetLastdigit()=='(') {
                oper = ' ';
                not = false;
            }
      
            MetricMatchingVariable mmvAppo=new MetricMatchingVariable();
            mmvAppo.setMatchingVariable(matchingVariable);
            mmvAppo.setComparisonFunction(comparisonFunction);
            mmvAppo.setMetricThreshold(threshold);
        
            currMV = mmvv.getIndex(mmvAppo);
      
            if (currMV<0) {
                currMV=indexMV;
                mmvv.add(indexMV, mmvAppo);
                indexMV++;
            }
      
            String newText = comparisonFunction+"["+matchingVariable+"]";
      
            if (!comparisonFunction.equals("Equality")) {
                 newText=newText+">="+threshold;
            }
       
            elementsRule[lastElement+1]=new RuleElement(lastElement+1,newText,
                                       elementsRule[lastElement].GetLevel(),
                                          currMV,
                                          'V',oper,not);
            lastElement++;
        //Fine Cose da fare quando finisce una regola (era add condition)
        }else if(token[i].equals(")")&& (token[i-2].equals(">=")||token[i-2].equals("Equality"))){
        //Cose da fare quando finisce una regola (era add condition)
            int currMV;
            if (elementsRule[lastElement].GetLastdigit() != '(') {
                elementsRule[lastElement+1]=new RuleElement(lastElement+1," And",
                                      elementsRule[lastElement].GetLevel(),
                                      -1,'O','A',false);
            lastElement++;
            }  
       
            char oper = elementsRule[lastElement].GetOperator();
            boolean not = elementsRule[lastElement].GetNot();
      
            if (elementsRule[lastElement].GetLastdigit()=='(') {
                oper = ' ';
                not = false;
            }
      
            MetricMatchingVariable mmvAppo=new MetricMatchingVariable();
            mmvAppo.setMatchingVariable(matchingVariable);
            mmvAppo.setComparisonFunction(comparisonFunction);
            mmvAppo.setMetricThreshold(threshold);
        
            currMV = mmvv.getIndex(mmvAppo);
      
            if (currMV<0) {
                currMV=indexMV;
                mmvv.add(indexMV, mmvAppo);
                indexMV++;
            }
      
            String newText = comparisonFunction+"["+matchingVariable+"]";
      
            if (!comparisonFunction.equals("Equality")) {
                 newText=newText+">="+threshold;
            }
            //System.out.println();
            //System.out.println(newText);
       
            elementsRule[lastElement+1]=new RuleElement(lastElement+1,newText,
                                       elementsRule[lastElement].GetLevel(),
                                          currMV,
                                          'V',oper,not);
            lastElement++;
        //Fine Cose da fare quando finisce una regola (era add condition)
        }else if (token[i].equalsIgnoreCase("OR")){
        //Cose da fare se la stringa letta e' OR (era or button)
            elementsRule[lastElement+1]=new RuleElement(lastElement+1,")",
                                      elementsRule[lastElement].GetLevel()-1,
                                      -1,')',' ',false);
            lastElement++;
    
              elementsRule[lastElement+1]=new RuleElement(lastElement+1," Or",
                                      elementsRule[lastElement].GetLevel(),
                                      -1,'O','O',false);
            lastElement++;
    
        //fine cose da fare se la stringa letta e' OR (era or button)
        } 
        if(token[i].equals(")")&& i+1==token.length){  
            //Cose da fare alla fine (era OK button) 
            
            elementsRule[lastElement+1]=new RuleElement(lastElement+1,")",
                                   elementsRule[lastElement].GetLevel()-1,
                                      -1,')',' ',false);
            lastElement++;
            elementsRule[lastElement+1]=new RuleElement(lastElement+1,"",
                                      elementsRule[lastElement].GetLevel(),
                                      -1,'E',' ',false);
            lastElement++;
            
            /*for (int dd=0;dd<(lastElement+1);dd++) {
                System.out.println("elem:"+dd+" dig:"+elementsRule[dd].lastdigit+" text:"+elementsRule[dd].text);
            }*/
            System.out.println("Matching variables found: "+mmvv.size());
            MatchingVariables mv = new MatchingVariables(connection,mmvv);
            mv.CreateMatchingVariablesTableComplete();
            //mv.UpdateMatchingVariableTable(mmvv);
            try{
              MUTableDeterministic mutable = new MUTableDeterministic(connection,indexMV,elementsRule);
              mutable.Load();
            }catch(Exception e){
                System.out.println(e);
            }
        
        //Fine cose da fare alla fine (era OK button) 
        }
       i++;
     }
   }  
    
   public void relaisBatchHelp() throws Exception{
        System.out.println(title);
        System.out.println("Parameters list:");
        System.out.println(" -------------- ");
        System.out.println("dsaFile");
        System.out.println("dsbFile");
        System.out.println("dsaSep");
        System.out.println("dsbSep");
        System.out.println("dedup");
        System.out.println("Method");
        System.out.println("BKey");
        System.out.println("SKey");
        System.out.println("WSize");
        System.out.println("SHParam");
        System.out.println("Model");
        //System.out.println("Rule");
        System.out.println("MVar");
        System.out.println("MMet");
        System.out.println("MThr");
        System.out.println("1to1");
        System.out.println("TU");
        System.out.println("TM");
        System.out.println("Folder");
   }
}
