package it.istat.relais.datasetInput;

import java.util.ArrayList;

public class Record {
	private final static int limit = -2;	
	private ArrayList<Object> tuple;
	
	/*Constructor*/
        public Record() {
		tuple = new ArrayList<Object>();
	}
	
	/*Constructor of the arraylist with initial length*/
        public Record(int initialCapacity) {
		tuple = new ArrayList<Object>(initialCapacity);
	}
	
	/*Constructor of the arraylist starting from a row 
         * and splitting it in variables by means of the separator*/
        public Record(String row, String separator) {
		String[] fields;
                String sep;
		if(separator.equals("|")){
                    sep = "\\|";
                }else
                    sep = separator;
                
                // parse a text line of the data set, splitting it in variables
		fields = getFields(row, sep);
		
		tuple = new ArrayList<Object>(fields.length);
		
		for (String field : fields){
                    add(field);
                }
	}
	
	/*Adds an object to the record*/
        public void add(Object obj) {
		tuple.add(obj);
	}
	
	/*Adds an object to the record in 
         * a specified position*/
        public void add(int index, Object obj) {
		tuple.add(index, obj);
	}
	
	/*Clears the arraylist*/
        public void clear() {
		tuple.clear();
	}
	
	/*Returns the object in a 
         * specified (index) position*/
        public Object get(int index) {
		return tuple.get(index);
	}
	
        /*Returns the position of
         * a specified object in the arraylist*/
	public int indexOf(Object obj) {
		return tuple.indexOf(obj);
	}
	
	/*Erase and returns the object in the 
         * specified (index) position*/
        public Object remove(int index) {
		return tuple.remove(index);
	}
	
	/*Erase the specified (obj) object*/
        public boolean remove(Object obj) {
		return tuple.remove(obj);
	}
	
	/*Returns the arraylist size*/
        public int size() {
		return tuple.size();
	}
	
	/*Transforms the arraylist into array*/
        public Object[] toArray() {
		return tuple.toArray();
	}
	
	/*Returns a string representation of the record*/
	public String toString(String separator) {
		StringBuilder row;
		
		if (size() < 1) return new String();
		
		row = new StringBuilder(String.valueOf(get(0)));
		for (int i = 1; i < size(); i++) {
			row.append(separator);
			row.append(get(i));
		}
		
		return row.toString();
	}
        
        /*Returns a specified (column) field of a row 
         * splitting the variables by means of a separator
         * character*/
	public static String getField(String row, String separator, int column) {
		return row.split(separator, limit)[column];
	}
	
	/*Returns an arraylist represention of a row
         * splitting it by means of a separator chararcter*/
        public static String[] getFields(String row, String separator) {
		return row.split(separator,limit);
	}
	
	/*Returns a parsed string representation of the input string. 
         * If there are two consecutive separators, a blank character is
         * introduced.
       */
        public static String parseRow(String row, String sep){
		String rowParsed="";
		for(int i=0; i<(row.length()-1); i++){
			if((row.charAt(i)==sep.charAt(0))&& (row.charAt(i+1)==sep.charAt(0)))
				rowParsed = rowParsed+sep+" ";	
			else rowParsed =rowParsed+row.charAt(i);
		}
		if((row.charAt(row.length()-1)==sep.charAt(0))) 
		    rowParsed = rowParsed+sep+" ";
		rowParsed = rowParsed+row.charAt(row.length()-1);
		return rowParsed;
	}	
}
