package it.istat.relais.deterministic;



import it.istat.relais.constants.TablesName;
import it.istat.relais.matchingVariables.MatchingVariables;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;

public class MUTableDeterministic {

private	Connection connection;
private int numVar;
private int dim;
private int[][] combinations; 	
private String[] patterns;
private RuleElement[] rulesElements;
private MatchingVariables mv;

/*Constructor that creates all the possible combinations
 *of patterns and creates all the necessary tables in the database*/
public MUTableDeterministic(Connection conn,int numberMM,RuleElement[] rule) throws Exception {
		this.connection=conn;
		this.numVar = numberMM;
                this.rulesElements=rule;
                this.mv = new MatchingVariables(conn);
                
               
                String pattern;
                try {		
                    double dim_d=Math.pow(2,numVar);
                    dim=(int) dim_d;
                    patterns = new String[dim];
                    combinations=new int[dim][numVar];
                    int i,j;
                    int nn=0;
                    int r=0;
                    for (i=0;i<dim;i++)
                    {
                            pattern ="";
                            nn=i;	
                            for (j=numVar;j>0;j--)
                            {r=nn%2;			
                             if (r==0) combinations[i][j-1]=0;
                             else combinations[i][j-1]=1;
                              nn=nn/2;	
                            }
                            for(int k=0; k<numVar;k++)
                                    pattern=pattern+combinations[i][k];
                            patterns[i]=pattern;
                    }

                    String[] cols=mv.getContingencyColumns();
                    String strCreate = "CREATE TABLE "+TablesName.nameMURule()+" (";
                    for (i=0;i<numVar;i++) {
                        strCreate = strCreate + cols[i] + " INTEGER,";
                    }
                    strCreate = strCreate + "RULE INTEGER, WEIGHT DOUBLE)";

                    Statement stmDrop =(Statement)connection.createStatement();
                    try {
                        stmDrop.executeUpdate("DROP TABLE "+TablesName.nameMURule());
                        stmDrop.close();
                    } catch (Exception ex) {

                    }

                    Statement stmCreate =(Statement)connection.createStatement();
                    stmCreate.executeUpdate(strCreate);
                    stmCreate.close();
                    
                    /*table of weigths*/
                    String strCreateW = "CREATE TABLE "+TablesName.nameReductionWeight()+" (SUBRULE VARCHAR(500), WEIGHT DOUBLE, ";
                    for (i=0;i<numVar;i++) {
                        strCreateW = strCreateW + cols[i] + " INTEGER,";
                    }
                    strCreateW = strCreateW + "ID_SR INTEGER)";

                    Statement stmDropW =(Statement)connection.createStatement();
                    try {
                        stmDropW.executeUpdate("DROP TABLE "+TablesName.nameReductionWeight());
                        stmDropW.close();
                    } catch (Exception ex) {

                    }

                    Statement stmCreateW =(Statement)connection.createStatement();
                    stmCreateW.executeUpdate(strCreateW);
                    stmCreateW.close();
                    
		}catch(Exception e){
			e.printStackTrace();
                        throw e;
		}
	}
	
/*Loads the rules in the table. Populates the table with 
 all the patterns and associates to each pattern a flag indicating if the rule is verified*/	
public int Load() throws Exception {
		try {
                        LoadTableWeight();
                        
                        Rule matchRule = new Rule(rulesElements);
			String pat;
			char rule;
			String strCT="";
			for(int i=0; i<this.dim; i++){
				strCT = "INSERT INTO "+TablesName.nameMURule()+" VALUES (";
				pat = patterns[i];
				rule = matchRule.Apply(pat);
				for(int j=0;j<this.numVar;j++)
					strCT = strCT+ pat.substring(j, j+1)+", ";
			strCT = strCT + rule+ ", 0);";
			Statement stm1= (Statement)connection.createStatement();	
			stm1.execute(strCT);
			stm1.close();
			}
			
                        Statement stm2= (Statement)connection.createStatement();	
			ResultSet rs = stm2.executeQuery("SELECT COUNT(*) FROM "+TablesName.nameMURule()+" WHERE RULE=1");
                        int nRuleVerify = 0;
                        if (rs.next()) {
                            nRuleVerify = rs.getInt(1);
                        }
                        stm2.close();
			if (nRuleVerify>0)
                            return 1;
                        
                        return 0;
	        
		}catch(Exception e){
			 e.printStackTrace();
			 throw e;
		}
		
	 }
 
/*Loads the default weights in the table. Defaults are computed as 0.5^x, where x is 
 order number of the subrule in the rule definition*/
	public void LoadTableWeight() throws Exception {
		try {
                        int order=0;
                        int index=1;
                        String currVar;
                        double weight;
                        String subRule="";
                        Statement stm1= (Statement)connection.createStatement();
                        
                        while (index<rulesElements.length && rulesElements[index].lastdigit!='E') {

                            if (rulesElements[index].lastdigit=='(') {
                                subRule="";
                                weight = Math.pow(0.5,order);
                                order ++;
                                stm1.executeUpdate("INSERT INTO "+TablesName.nameReductionWeight()+" (WEIGHT, ID_SR) VALUES ("+weight+","+order+")");
                            }
                            else if (rulesElements[index].lastdigit==')') {
                                stm1.executeUpdate("UPDATE "+TablesName.nameReductionWeight()+" SET SUBRULE='"+subRule+"' WHERE ID_SR="+order);
                            }
                            else {
                                subRule = subRule + rulesElements[index].GetText();
                                if (rulesElements[index].lastdigit=='V') {
                                    currVar=mv.getContingencyColumns()[rulesElements[index].variable];
                                    stm1.executeUpdate("UPDATE "+TablesName.nameReductionWeight()+" SET "+currVar+"=1 WHERE ID_SR="+order);
                                }
                            }
                            index++;
			}
	        
		}catch(Exception e){
			 e.printStackTrace();
			 throw e;
		}
		
	 }
}






