package it.istat.relais.export;

import it.istat.relais.constants.TablesName;
import it.istat.relais.project.ReconciledSchema;
import it.istat.relais.project.VariablesSetting;
import java.io.File;
import java.io.FileWriter;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;

public class ExportToFile {
private final int DIMMAX = 20000;
File fileOut;
FileWriter streem;
Connection conn;
String pathFile;
String separator;
VariablesSetting sett;

/*Constructor*/
public ExportToFile(String pathFile, Connection conn) {
	this.conn=conn;
	this.pathFile = pathFile;
        sett = new VariablesSetting(conn);
        this.separator=sett.ReadSeparator();
}

/*Returns the file name of the output file*/
public File getFile() {
	return fileOut;
}

/*Exports a table to a file*/
public void expTable(String fileName, String schema, String table,boolean skipFirstColumn) throws Exception {
	
	int nColumns=0;
        int colStart=1;
        if (skipFirstColumn) colStart=2;
	this.fileOut = new File(pathFile+System.getProperty("file.separator")+fileName);
	this.streem = new FileWriter(fileOut, false);
	
	String queryData = "SELECT * FROM "+schema+"."+table.toUpperCase();
	String queryStruct = "SELECT COLUMN_NAME FROM information_schema.COLUMNS" +
		" WHERE UPPER(TABLE_SCHEMA)='"+schema.toUpperCase()+"' AND UPPER(TABLE_NAME)='"+table.toUpperCase()+"'"+
		" ORDER BY ORDINAL_POSITION;";
        
               
	Statement stColumns = (Statement) conn.createStatement();
	ResultSet rsColumns = stColumns.executeQuery(queryStruct);
	
        if (skipFirstColumn) {
            rsColumns.next();
            nColumns++;
        }
        
	while (rsColumns.next()) {
		if(nColumns < colStart)
		streem.write(rsColumns.getString(1));
		else
		streem.write(separator+rsColumns.getString(1));
		nColumns++;
	}
	streem.write(System.getProperty("line.separator"));
	rsColumns.close();
	stColumns.close();
    
	Statement stValues = (Statement) conn.createStatement();
	ResultSet rsValues = stValues.executeQuery(queryData);
	
	while (rsValues.next()) {
		for (int i=1;i<=nColumns;i++) {
			if (i==colStart)
				streem.write(rsValues.getString(i));	
			else if (i>colStart)
				streem.write(separator+rsValues.getString(i));
		}
		streem.write(System.getProperty("line.separator"));
		
	}
	rsValues.close();
	stValues.close();
	streem.flush();
        streem.close();
}

public void expTable(String fileName, String table) throws Exception {	
	this.expTable(fileName, "relais",table,false);
}

public void expTableDS(String fileName, String table) throws Exception {
 	this.expTable(fileName, "relais",table,true);
}

public void close() throws Exception {
	this.streem.close();
}

/*Exports the match, possible match and the unmatch tables*/
public void expTableMPU(String fileName, String schema, String table) throws Exception {
	
	int nColumnsDS=0;
	int rowloaded=0,rowtoload;
        ReconciledSchema rsc = new ReconciledSchema(conn);
        String model=sett.ReadModel();
        this.fileOut = new File(pathFile+System.getProperty("file.separator")+fileName);
	
       
        this.streem = new FileWriter(fileOut, false);
	
	String queryData = "SELECT * FROM "+schema+"."+table.toUpperCase();
	
	String[] commVar = rsc.getReconciledSchema();
	
	streem.write("DS"+separator+"KEY_DS");

	while (nColumnsDS<commVar.length) {
		streem.write(separator);
		streem.write(commVar[nColumnsDS]);
		nColumnsDS++;
	}
        if (!model.startsWith("Deterministic"))
            streem.write(separator+"R"+separator+"P_POST");
	streem.write(System.getProperty("line.separator"));
    
	String selectCount = "SELECT MAX(ROWNUMBER) FROM "+schema+"."+table.toUpperCase();
	Statement stmCount = (Statement)conn.createStatement();			
	ResultSet rsCount = stmCount.executeQuery(selectCount);
	rsCount.next();
	rowtoload =  rsCount.getInt(1);
	rsCount.close();
	stmCount.close();

	Statement stValues;
	String insertStringDM;
	
	while (rowloaded < rowtoload) 
	{
		insertStringDM = queryData;
		insertStringDM = insertStringDM + " WHERE ROWNUMBER > "+ rowloaded;
		if (rowtoload>(rowloaded+DIMMAX))
			insertStringDM = insertStringDM + " AND ROWNUMBER <= "+ (rowloaded+DIMMAX);
		stValues = (Statement) conn.createStatement();
		ResultSet rsValues = stValues.executeQuery(insertStringDM);
	
		while (rsValues.next()) {
			streem.write("A"); 
			for (int i=1;i<=(1+nColumnsDS);i++) {
				streem.write(separator+rsValues.getString(i));
			}
                        if (!model.startsWith("Deterministic"))
                            streem.write(separator+rsValues.getString(3+2*nColumnsDS)+separator+rsValues.getString(4+2*nColumnsDS)); /*r-ppost*/
			streem.write(System.getProperty("line.separator"));
		
			streem.write("B"); 
			for (int i=nColumnsDS+2;i<=(2+2*nColumnsDS);i++) {
				streem.write(separator+rsValues.getString(i));
			}
                        if (!model.startsWith("Deterministic"))
                            streem.write(separator+rsValues.getString(3+2*nColumnsDS)+separator+rsValues.getString(4+2*nColumnsDS)); /*r-ppost*/
			streem.write(System.getProperty("line.separator"));
		}
		rsValues.close();
		stValues.close();
		rowloaded = rowloaded + DIMMAX;
	}
	streem.flush();
        streem.close();
}

public void expTableMPU(String fileName,String table) throws Exception {
       this.expTableMPU(fileName,"relais",table);
       
}

}