package it.istat.relais.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SpringLayout;

public class ChooseVariable extends JDialog {
	
	public static final int UNINITIALIZED_EXIT_STATUS = -1;
	public static final int OK_EXIT_STATUS = 0;
	public static final int CANCEL_EXIT_STATUS = 1;
	public static final int CLOSED_EXIT_STATUS = 2;
	private static final long serialVersionUID = 1L;
	public JButton okButton, cancelButton;
	private ListPane listPane;
	private Object[] chosenItems = new Object[0];
	private int exitStatus;
	
	public ChooseVariable(Frame owner, String itemLabel, String chosenItemLabel) {
		super(owner);
		
		SpringLayout centerLayout = new SpringLayout();
		SpringLayout southLayouts = new SpringLayout();
		JPanel centerFreezer = new JPanel(centerLayout);
		JPanel southFreezer = new JPanel(southLayouts);
		getContentPane().add(centerFreezer, BorderLayout.CENTER);
		getContentPane().add(southFreezer, BorderLayout.SOUTH);
		
		listPane = new ListPane(itemLabel, chosenItemLabel);
		JPanel southPanel = createSouthPanel();
		centerFreezer.add(listPane);
		southFreezer.add(southPanel);
		
		centerLayout.putConstraint(SpringLayout.WEST, listPane,
								   0,
								   SpringLayout.WEST, centerFreezer);
		centerLayout.putConstraint(SpringLayout.NORTH, listPane,
								   0,
								   SpringLayout.NORTH, centerFreezer);
		centerLayout.putConstraint(SpringLayout.EAST, centerFreezer,
								   0,
								   SpringLayout.EAST, listPane);
		centerLayout.putConstraint(SpringLayout.SOUTH, centerFreezer,
								   0,
								   SpringLayout.SOUTH, listPane);
		
		southLayouts.putConstraint(SpringLayout.WEST, southPanel,
								   0,
								   SpringLayout.WEST, southFreezer);
		southLayouts.putConstraint(SpringLayout.NORTH,southPanel,
								   0,
								   SpringLayout.NORTH, southFreezer);
		southLayouts.putConstraint(SpringLayout.EAST, southFreezer,
								   0,
								   SpringLayout.EAST, southPanel);
		southLayouts.putConstraint(SpringLayout.SOUTH, southFreezer,
								   0,
								   SpringLayout.SOUTH, southPanel);
		
		WindowListener listener = new MyWindowListener();
		addWindowListener(listener);
		
		freezeSize(listPane, southPanel);
		int width = (int) (getSize().width * 1.5);
		int height = (int) (getSize().height * 1.5);
		setSize(new Dimension(width, height));
		
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = getSize();
		setLocation((screenSize.width - frameSize.width) / 2,
					(screenSize.height - frameSize.height) / 2);
	}
	
	public int getExitStatus() {
		return exitStatus;
	}
	
	public Object[] getChosenItems() {
		return chosenItems;
	}
	
	public Object[] getItems() {
		return listPane.getListData();
	}
	
	public void setItems(Object[] items) {
		listPane.setListData(items);
		chosenItems = new Object[0];
	}
	

	public void clearChosenItems(){
		chosenItems = new Object[0];
		listPane.clearListData();
		
	}
	
	
	private JPanel createSouthPanel() {
		JPanel southPanel;
		BagBuilder bag;
		ActionListener okCancel_ButtonListener = new OkCancel_ButtonListener();
		
		southPanel = new JPanel();
		bag = new BagBuilder(southPanel);
		bag.constraints().insets = new Insets(10, 10, 10, 10);
		
		okButton = new JButton("Ok");
		okButton.addActionListener(okCancel_ButtonListener);
		bag.add(okButton, 0, 0, 1, 1, 1, 0,
				GridBagConstraints.NONE,
				GridBagConstraints.EAST);
		
		cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(okCancel_ButtonListener);
		bag.add(cancelButton, 1, 0, 1, 1, 0, 0,
				GridBagConstraints.NONE,
				GridBagConstraints.CENTER);
		
		return southPanel;
	}
	
	private void freezeSize(Component... components) {
		pack();
		for (Component component : components) {
			component.setPreferredSize(component.getSize());
			component.setMinimumSize(component.getSize());
			component.setMaximumSize(new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE));
		}
	}
	
	private boolean isComplete() {
		return listPane.getChosenListData().length > 0;
	}
	
	private boolean getWidget() {
		if (!isComplete()) {
			JOptionPane.showMessageDialog(this,
										  "Missing parameters",
										  "Warning",
										  JOptionPane.WARNING_MESSAGE);
			return false;
		}
		
		chosenItems = listPane.getChosenListData();
		return true;
	}
	
	private void setWidget() {
		listPane.setListData(listPane.getListData(), chosenItems);
	}
	
	private class OkCancel_ButtonListener implements ActionListener {
		
		public void actionPerformed(ActionEvent event) {
			Object src = event.getSource();
			
			if (src == okButton) {
				if (getWidget()) {
					exitStatus = OK_EXIT_STATUS;
					setVisible(false);
				}
			}
			else if (src == cancelButton) {
				setWidget();
				exitStatus = CANCEL_EXIT_STATUS;
				setVisible(false);
			}
		}
		
	}
	
	private class MyWindowListener extends WindowAdapter {
		
		public void windowClosing(WindowEvent e) {
			setWidget();
			exitStatus = CLOSED_EXIT_STATUS;
		}
		
		public void windowOpened(WindowEvent e) {
			exitStatus = UNINITIALIZED_EXIT_STATUS;
		}
		
	}
	
}
