package it.istat.relais.gui;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

public class ListPane extends JPanel {
	
	private static final long serialVersionUID = 1L;
	private JLabel label1, label2;
	private JScrollPane scrollPane1, scrollPane2;
	private JButton addButton, removeButton;
	private DefaultListModel listModel, chosenListModel;
	private JList list, chosenList;
	private Object[] listData = new Object[0];
	
	public ListPane(String listLabel, String chosenListLabel) {
		BagBuilder bag;
		ActionListener listener = new ButtonListener();
		
		bag = new BagBuilder(this);
		bag.constraints().insets = new Insets(10, 10, 10, 10);
		
		label1 = new JLabel(listLabel);
		bag.add(label1, 0, 0, 1, 1, 0, 0,
				GridBagConstraints.NONE,
				GridBagConstraints.WEST);
		
		label2 = new JLabel(chosenListLabel);
		bag.add(label2, 2, 0, 1, 1, 0, 0,
				GridBagConstraints.NONE,
				GridBagConstraints.WEST);
		
		listModel = new DefaultListModel();
		list = new JList(listModel);
		scrollPane1 = new JScrollPane(list);
		bag.add(scrollPane1, 0, 1, 1, 2, 1, 1,
				GridBagConstraints.BOTH,
				GridBagConstraints.CENTER);
		
		chosenListModel = new DefaultListModel();
		chosenList = new JList(chosenListModel);
		scrollPane2 = new JScrollPane(chosenList);
		bag.add(scrollPane2, 2, 1, 1, 2, 1, 1,
				GridBagConstraints.BOTH,
				GridBagConstraints.CENTER);
		
		addButton = new JButton("Add");
		addButton.addActionListener(listener);
		bag.add(addButton, 1, 1, 1, 1, 0, 0,
				GridBagConstraints.NONE,
				GridBagConstraints.CENTER);
		
		removeButton = new JButton("Remove");
		removeButton.addActionListener(listener);
		bag.add(removeButton, 1, 2, 1, 1, 0, 0,
				GridBagConstraints.NONE,
				GridBagConstraints.NORTH);
		
		widgetSizing();
	}
	
	public Object[] getChosenListData() {
		return chosenListModel.toArray();
	}
	
	public Object[] getListData() {
		return listData;
	}
	
	public void setListData(Object[] data) {
		setListData(data, new Object[0]);
	}
	
	public void setListData(Object[] data, Object[] chosen) {
		listModel.clear();
		chosenListModel.clear();
		for (Object obj : data)
			listModel.addElement(obj);
		
		for (Object obj : chosen) {
			listModel.removeElement(obj);
			chosenListModel.addElement(obj);
		}
		
		listData = data;
	}
	
	private void widgetSizing() {
		int maxLabelWidth;
		
		maxLabelWidth = Math.max(label1.getPreferredSize().width,
								 label2.getPreferredSize().width);
		
		label1.setPreferredSize(new Dimension(maxLabelWidth,
											  label1.getPreferredSize().height));
		label1.setMinimumSize(label1.getPreferredSize());
		
		label2.setPreferredSize(new Dimension(maxLabelWidth,
											  label2.getPreferredSize().height));
		label2.setMinimumSize(label2.getPreferredSize());
		
		scrollPane1.setPreferredSize(new Dimension(maxLabelWidth,
												   scrollPane1.getPreferredSize().height));
		scrollPane1.setMinimumSize(scrollPane1.getPreferredSize());
		
		scrollPane2.setPreferredSize(new Dimension(maxLabelWidth,
												   scrollPane2.getPreferredSize().height));
		scrollPane2.setMinimumSize(scrollPane2.getPreferredSize());
		
		addButton.setPreferredSize(removeButton.getPreferredSize());
		addButton.setMinimumSize(removeButton.getMinimumSize());
		
		removeButton.setPreferredSize(removeButton.getPreferredSize());
		removeButton.setMinimumSize(removeButton.getMinimumSize());
	}
	
	private class ButtonListener implements ActionListener {
		
		public void actionPerformed(ActionEvent event) {
			Object src = event.getSource();
			int i, j;
			
			if (src == addButton) {
				for (Object obj : list.getSelectedValues()) {
					chosenListModel.addElement(obj);
					listModel.removeElement(obj);
				}
			}
			else if (src == removeButton) {
				for (Object obj : chosenList.getSelectedValues()) {
					chosenListModel.removeElement(obj);
					
					i = j = 0;
					while (listData[i] != obj && j < listModel.size())
						if (listData[i++] == listModel.get(j))
							j++;
					
					listModel.add(j, obj);
				}
			}
		}
		
	}
	
	public void clearListData() {
		Object obj;
		int i,j;
		while (chosenListModel.getSize()!=0){
			
			obj = chosenListModel.elementAt(0);
			chosenListModel.removeElement(obj);
			
			i = j = 0;
			while (listData[i] != obj && j < listModel.size())
				if (listData[i++] == listModel.get(j))
					j++;
			
			listModel.add(j, obj);
		
	}
	}
}
