package it.istat.relais.gui;

import java.io.OutputStream;
import java.io.PrintStream;
import java.nio.ByteBuffer;
import java.nio.charset.Charset;

import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

public class OutScrollPane extends JScrollPane {
	
	private static final long serialVersionUID = 1L;
	private JTextArea textArea;
	private PrintStream out = new PrintStream(new OutScrollPaneStream());
	private Charset charset = Charset.defaultCharset();
	
	public OutScrollPane() {
		textArea = new JTextArea();
		textArea.setEditable(false);
		
		getViewport().add(textArea);
	}
	
	public PrintStream getOutputStream() {
		return out;
	}
	
	public void append(final String str) {
		Runnable append = new Runnable() {
			public void run() {
				textArea.append(str);
				getVerticalScrollBar().setValue(textArea.getPreferredScrollableViewportSize().height);
			}
		};
		SwingUtilities.invokeLater(append);
	}
	
	private synchronized void append(ByteBuffer buffer) {
		final String str = charset.decode(buffer).toString();
		Runnable append = new Runnable() {
			public void run() {
				textArea.append(str);
				getVerticalScrollBar().setValue(textArea.getPreferredScrollableViewportSize().height);
			}
		};
		SwingUtilities.invokeLater(append);
	}
	
        public void clearAll(){
                textArea.setText("");
        }
                        
	private class OutScrollPaneStream extends OutputStream {
		
		private ByteBuffer buffer = ByteBuffer.allocate(1024);
		
		public synchronized void write(int b) {
			buffer.put((byte) b);
			append(buffer);
			buffer.clear();
		}
		
		public synchronized void write(byte[] b) {
			if(b.length < buffer.capacity()) {
				buffer.put(b);
				buffer.flip();
				append(buffer);
				buffer.clear();
			}
		}
		
		public synchronized void write(byte[] b, int off, int len) {
			if(len < buffer.capacity()) {
				for(int k = off; k < len; k++) {
					buffer.put(b[k]);
					buffer.flip();
					append(buffer);
					buffer.clear();
				}
			}
		}
                
		
		public void flush() {}
		
		public void close() {}
		
	};
	
}
