package it.istat.relais.gui.task;

import it.istat.relais.decision.contingencyTable.ContingencyTable;
import java.sql.Connection;
import it.istat.relais.log.TraceLog;
import it.istat.relais.result.ResultTable;
import it.istat.relais.project.VariablesSetting;

public class ContingencyTableTask extends AbstractTask {

    private Connection conn;
    private String method;
    private String model;
    private VariablesSetting vs;
    private String orderKey;
    
    public ContingencyTableTask(Connection conn, String method) {
        this.conn=conn;
        this.method=method;
        this.vs=null;
        this.model="Fellegi-Sunter";
    }
    public ContingencyTableTask(Connection conn, String method, String model, VariablesSetting vs, String orderKey) {
        this.conn=conn;
        this.method=method;
        this.vs=vs;
        this.model=model;
        this.orderKey = orderKey;
    }
    public void run() {
        TraceLog log = new TraceLog(conn);
        try {
            ContingencyTable ctg;
            ctg = new ContingencyTable(conn);
            log.AddMessage("Evaluation of Contingency table. Model: "+model+"Method: "+method,false);
            if (method.endsWith("Blocking")) {
                ctg.CreateContingencyTableBlocking();
            } else if (method.equals("Exact")) {
                log.AddMessage("Exact Matching does not need contingency table","I",false);
            }  else {
                ctg.CreateContingencyTable();            
            }
            log.AddMessage("Update pattern and contingency table creation done",false);
            if (model.equals("Deterministic")) {
                vs.UpdateThresholdU(0.5);
                vs.UpdateThresholdM(1);
                log.AddMessage("Automatic setting of U/M tresholds",false);
                
                if (method.equals("Exact")) {
                    ResultTable rt = new ResultTable(conn);
                    rt.CreateTableMatch(this.orderKey);
                    log.AddMessage("Match table created",false);
                    rt.CreateTableUnmatchDSA();
                    log.AddMessage("Residual dsa table created", false);
                    rt.CreateTableUnmatchDSB();
                    log.AddMessage("Residual dsb table created", false);
                }
            }
            succeed();
        } catch (Exception ex) {
            try {
                log.AddMessage("Exception in update pattern","E",false);
                log.AddMessage(ex.getMessage(),"E",false);
            } catch (Exception ex2) { 
                System.out.println(ex2.getMessage());
                System.out.println(ex.getMessage());
            } finally {
                fail();
            }
        }
    }
}
