package it.istat.relais.gui.task;

import it.istat.relais.constants.TablesName;
import it.istat.relais.displayTables.DisplayTable;
import it.istat.relais.gui.ChooseMetadataParametersBlocking;
import it.istat.relais.gui.ChooseMetadataParametersMatching;
import it.istat.relais.gui.WaitDialog;
import it.istat.relais.log.TraceLog;
import it.istat.relais.project.ReconciledSchema;
import it.istat.relais.qualityMetaData.*;
import java.sql.Connection;

public class DataProfilingTask extends AbstractTask {

    private Connection conn;
    private String profiletype;
    Object[] varlist;
    WaitDialog wait;
    javax.swing.JFrame gui;
    String menuDisplay;
    
    public DataProfilingTask(Connection conn, String profiletype, Object[] varlist, WaitDialog wait, javax.swing.JFrame gui, String menuDisplay) {
        this.conn=conn;
        this.profiletype=profiletype;
        this.varlist=varlist;
        this.wait=wait;
        this.gui=gui;
        this.menuDisplay=menuDisplay;
    }
    public void run() {
        String selvar;
        String message;
        boolean show=false;
        TraceLog log = new TraceLog(conn);
        ReconciledSchema rs = new ReconciledSchema(conn);
        MetaData  md = new MetaData(conn);
        String type="Setting";
        String selvara,selvarb;
        
        try{
            log.AddMessage("Evaluation of data profiling type: "+profiletype,false);
            if (profiletype.equalsIgnoreCase("blockvar")) {
                md.CreateBlockingMetaDataTable();
                for (int numvar=0; numvar<varlist.length;numvar++) {
                    selvar = (String) varlist[numvar];                

                    ChooseMetadataParametersBlocking cmp = new ChooseMetadataParametersBlocking(gui,true);
                    cmp.setVariableName(selvar);
                    cmp.setVisible(true);

                    if (cmp.getReturnStatus()==ChooseMetadataParametersBlocking.RET_OK) {
                       selvara = rs.getAttribute(selvar,"A");
                       selvarb = rs.getAttribute(selvar,"B");
                        wait.setVisible(true);

                        message="";

                         if (cmp.getCompleteness()) {
                             md.InsertRowBlockingMetaDataTable("DSA","completeness", selvar);
                             Completeness cplDSA = new Completeness(conn, TablesName.nameDsa(), md.getDimDSA(), selvara);
                             md.UpdateRowBlockingMetaDataTable("DSA", "completeness", cplDSA.EvaluateCompleteness(),selvar);
                             
                             md.InsertRowBlockingMetaDataTable("DSB","completeness", selvar);
                             Completeness cplDSB = new Completeness(conn, TablesName.nameDsb(), md.getDimDSB(), selvarb);
                             md.UpdateRowBlockingMetaDataTable("DSB", "completeness", cplDSB.EvaluateCompleteness(),selvar);
                             
                             message=message+" Completeness";
                                type="Completeness";
                            }
                            if (cmp.getAccuracy()) {
                             
                             md.InsertRowBlockingMetaDataTable("DSA","accuracy", selvar);
                             Accuracy accDSA = new Accuracy(conn, TablesName.nameDsa(), md.getDimDSA(), selvara);
                             md.UpdateRowBlockingMetaDataTable("DSA", "accuracy", accDSA.EvaluateAccuracy(cmp.getAccuracyFile(), cmp.getAccuracySep()),selvar);
                             
                             md.InsertRowBlockingMetaDataTable("DSB","accuracy", selvar);
                             Accuracy accDSB = new Accuracy(conn, TablesName.nameDsb(), md.getDimDSB(), selvarb);
                             md.UpdateRowBlockingMetaDataTable("DSB", "accuracy", accDSB.EvaluateAccuracy(cmp.getAccuracyFile(), cmp.getAccuracySep()),selvar);
                             
                             message=message+" Accuracy";
                                type="Accuracy";
                            }
                            if (cmp.getConsistency()) {
                             
                             md.InsertRowBlockingMetaDataTable("DSA","consistency", selvar);
                             Consistency conDSA = new Consistency(conn, TablesName.nameDsa(), md.getDimDSA(), selvara);
                             md.UpdateRowBlockingMetaDataTable("DSA", "consistency", conDSA.EvaluateConsistency(cmp.getConsistencyFile(), cmp.getConsistencySep()),selvar);
                             
                             md.InsertRowBlockingMetaDataTable("DSB","consistency", selvar);
                             Consistency conDSB = new Consistency(conn, TablesName.nameDsb(), md.getDimDSB(), selvarb);
                             md.UpdateRowBlockingMetaDataTable("DSB", "consistency", conDSB.EvaluateConsistency(cmp.getConsistencyFile(), cmp.getConsistencySep()),selvar);
                             
                             message=message+" Consistency";
                                type="Consistency";
                            }
                            
                            if (cmp.getCategories()) {
                             
                             md.InsertRowBlockingMetaDataTable("DSA","categories", selvar);
                             Categories ctDSA = new Categories(conn, TablesName.nameDsa(), selvara);
                             md.UpdateRowBlockingMetaDataTable("DSA", "categories", ctDSA.EvaluateCategories(),selvar);

                             md.InsertRowBlockingMetaDataTable("DSB","categories", selvar);
                             Categories ctDSB = new Categories(conn, TablesName.nameDsb(), selvarb);
                             md.UpdateRowBlockingMetaDataTable("DSB", "categories", ctDSB.EvaluateCategories(),selvar);

                             message=message+" Categories";
                                type="Categories";
                            }
                            
                           if (cmp.getFrequencyDistributions()) {
                             
                             md.InsertRowBlockingMetaDataTable("DSA","frequencyDistributions (see Output Tables for computed values)", selvar);
                             FrequencyDistributions fdDSA = new FrequencyDistributions(conn, TablesName.nameDsa(), selvara);
                             fdDSA.setDisplayMenu(menuDisplay);
                             fdDSA.EvaluateFrequencyDistributions();
                            
                             md.InsertRowBlockingMetaDataTable("DSB","frequencyDistributions (see Output Tables for computed values)", selvar);
                             FrequencyDistributions fdDSB = new FrequencyDistributions(conn, TablesName.nameDsb(), selvarb);
                             fdDSB.setDisplayMenu(menuDisplay);
                             fdDSB.EvaluateFrequencyDistributions();
                            
                             message=message+" FrequencyDistributions";
                                type="FrequencyDistributions";
                            }
                            if (cmp.getBlockingAdeguacy()) {
                             int blockDim = Integer.parseInt(cmp.getBlockAdeguacydimention());  
                             md.InsertRowBlockingMetaDataTable("DSA","blockingAdequacy", selvar);
                             BlockingAdeguacy baDSA = new BlockingAdeguacy(conn, TablesName.nameDsa(), selvara, blockDim);
                             md.UpdateRowBlockingMetaDataTable("DSA", "blockingAdequacy", baDSA.EvaluateBlockingAdeguacy(),selvar);

                             md.InsertRowBlockingMetaDataTable("DSB","blockingAdequacy", selvar);
                             BlockingAdeguacy baDSB = new BlockingAdeguacy(conn, TablesName.nameDsb(), selvarb, blockDim);
                             md.UpdateRowBlockingMetaDataTable("DSB", "blockingAdequacy", baDSB.EvaluateBlockingAdeguacy(),selvar);

                             message=message+" BlockingAdequacy";
                                type="BlockingAdequacy";
                            }
                          
                            if (cmp.getEntropy()) {
                             md.InsertRowBlockingMetaDataTable("DSA","entropy", selvar);
                             Entropy eDSA = new Entropy(conn, TablesName.nameDsa(), selvara);
                             md.UpdateRowBlockingMetaDataTable("DSA", "entropy", eDSA.EvaluateEntropy(),selvar);

                             md.InsertRowBlockingMetaDataTable("DSB","entropy", selvar);
                             Entropy eDSB = new Entropy(conn, TablesName.nameDsb(), selvarb);
                             md.UpdateRowBlockingMetaDataTable("DSB", "entropy", eDSB.EvaluateEntropy(),selvar);
                                
                                message=message+" Entropy";
                                type="Entropy";
                            }
                            
                            log.AddMessage("Metadata for variable "+selvar+": "+message);
             
                        wait.setVisible(false);
                        show = true;
                    }
                }

                if (show) {
                    log.AddMessage("Display data from table "+TablesName.nameBlockingMetaData());
                    DisplayTable dt = new DisplayTable(conn, TablesName.nameBlockingMetaData());
                }
            }
            
            if (profiletype.equalsIgnoreCase("matchingvar")) {
                md.CreateMatchingMetaDataTable();
                for (int numvar=0; numvar<varlist.length;numvar++) {
                    selvar = (String) varlist[numvar];                

                    ChooseMetadataParametersMatching cmp = new ChooseMetadataParametersMatching(gui,true,this.conn);
                    cmp.setVariableName(selvar);
                    cmp.setVisible(true);
                    if (cmp.getReturnStatus()==ChooseMetadataParametersBlocking.RET_OK) {
                        selvara = rs.getAttribute(selvar,"A");
                        selvarb = rs.getAttribute(selvar,"B");
                        wait.setVisible(true);

                        message="";


                         if (cmp.getCompleteness()) {
                             md.InsertRowMatchingMetaDataTable("DSA","completeness", selvar);
                             Completeness cplDSA = new Completeness(conn, TablesName.nameDsa(), md.getDimDSA(), selvara);
                             md.UpdateRowMatchingMetaDataTable("DSA", "completeness", cplDSA.EvaluateCompleteness(),selvar);

                             md.InsertRowMatchingMetaDataTable("DSB","completeness", selvar);
                             Completeness cplDSB = new Completeness(conn, TablesName.nameDsb(), md.getDimDSB(), selvarb);
                             md.UpdateRowMatchingMetaDataTable("DSB", "completeness", cplDSB.EvaluateCompleteness(),selvar);

                             message=message+" Completeness";
                                type="Completeness";
                            }
                            if (cmp.getAccuracy()) {

                             md.InsertRowMatchingMetaDataTable("DSA","accuracy", selvar);
                             Accuracy accDSA = new Accuracy(conn, TablesName.nameDsa(), md.getDimDSA(), selvara);
                             md.UpdateRowMatchingMetaDataTable("DSA", "accuracy", accDSA.EvaluateAccuracy(cmp.getAccuracyFile(), cmp.getAccuracySep()),selvar);

                             md.InsertRowMatchingMetaDataTable("DSB","accuracy", selvar);
                             Accuracy accDSB = new Accuracy(conn, TablesName.nameDsb(), md.getDimDSB(), selvarb);
                             md.UpdateRowMatchingMetaDataTable("DSB", "accuracy", accDSB.EvaluateAccuracy(cmp.getAccuracyFile(), cmp.getAccuracySep()),selvar);

                             message=message+" Accuracy";
                                type="Accuracy";
                            }
                            if (cmp.getConsistency()) {

                             md.InsertRowMatchingMetaDataTable("DSA","consistency", selvar);
                             Consistency conDSA = new Consistency(conn, TablesName.nameDsa(), md.getDimDSA(), selvara);
                             md.UpdateRowMatchingMetaDataTable("DSA", "consistency", conDSA.EvaluateConsistency(cmp.getConsistencyFile(), cmp.getConsistencySep()),selvar);

                             md.InsertRowMatchingMetaDataTable("DSB","consistency", selvar);
                             Consistency conDSB = new Consistency(conn, TablesName.nameDsb(), md.getDimDSB(), selvarb);
                             md.UpdateRowMatchingMetaDataTable("DSB", "consistency", conDSB.EvaluateConsistency(cmp.getConsistencyFile(), cmp.getConsistencySep()),selvar);

                             message=message+" Consistency";
                                type="Consistency";
                            }


                           if (cmp.getFrequencyDistributions()) {

                             md.InsertRowMatchingMetaDataTable("DSA","frequencyDistributions (see Output Tables for computed values)", selvar);
                             FrequencyDistributions fdDSA = new FrequencyDistributions(conn, TablesName.nameDsa(), selvara);
                             fdDSA.setDisplayMenu(menuDisplay);
                             fdDSA.EvaluateFrequencyDistributions();

                             md.InsertRowMatchingMetaDataTable("DSB","frequencyDistributions (see Output Tables for computed values)", selvar);
                             FrequencyDistributions fdDSB = new FrequencyDistributions(conn, TablesName.nameDsb(), selvarb);
                             fdDSB.setDisplayMenu(menuDisplay);
                             fdDSB.EvaluateFrequencyDistributions();

                             message=message+" FrequencyDistributions";
                                type="FrequencyDistributions";
                            }

                          if (cmp.getCorrelationCramerIndex()) {
                              double ci;
                              md.InsertRowMatchingMetaDataTable("DSA","correlation", selvar);
                             CramerIndex cDSA = new CramerIndex(conn, TablesName.nameDsa(), md.getDimDSA(), selvara, rs.getAttribute(cmp.getCorrelationVarName2(),"A"));
                             double nCIA = cDSA.EvaluateCramerIndex();
                             if (nCIA==-1)
                                 System.out.println("Correlation not evaluated: the variable "+selvar+" or "+cmp.getCorrelationVarName2()+" has only one modality");
                             else 
                                 md.UpdateRowMatchingMetaDataTable("DSA", "correlation", nCIA, selvar);

                             md.InsertRowMatchingMetaDataTable("DSB","correlation", selvar);
                             CramerIndex cDSB = new CramerIndex(conn, TablesName.nameDsb(), md.getDimDSB(), selvarb, rs.getAttribute(cmp.getCorrelationVarName2(),"B"));
                             double nCIB = cDSB.EvaluateCramerIndex();
                             if (nCIB==-1)
                                 System.out.println("Correlation not evaluated: the variable "+selvar+" or "+cmp.getCorrelationVarName2()+" has only one modality");
                             else 
                                md.UpdateRowMatchingMetaDataTable("DSB", "correlation", nCIB, selvar);

                                message=message+" Correlation";
                                type="Correlation";
                            }

                            if (cmp.getEntropy()) {
                             md.InsertRowMatchingMetaDataTable("DSA","entropy", selvar);
                             Entropy eDSA = new Entropy(conn, TablesName.nameDsa(), selvara);
                             md.UpdateRowMatchingMetaDataTable("DSA", "entropy", eDSA.EvaluateEntropy(),selvar);

                             md.InsertRowMatchingMetaDataTable("DSB","entropy", selvar);
                             Entropy eDSB = new Entropy(conn, TablesName.nameDsb(), selvarb);
                             md.UpdateRowMatchingMetaDataTable("DSB", "entropy", eDSB.EvaluateEntropy(),selvar);

                                message=message+" Entropy";
                                type="Entropy";
                            }

                            log.AddMessage("Metadata for variable "+selvar+": "+message);

                        wait.setVisible(false);
                        show = true;
                    }
                }

                if (show) {
                    log.AddMessage("Display data from table "+TablesName.nameMatchingMetaData());
                    DisplayTable dt = new DisplayTable(conn, TablesName.nameMatchingMetaData());
                }
            }
            
            succeed();
            }
            catch (Exception ex) {
                        try {
                            log.AddMessage("Exception on meta data evaluetion ("+type+")","E",false);
                            log.AddMessage(ex.getMessage(),"E",false);
                        } catch (Exception ex2) { 
                            System.out.println(ex2.getMessage());
                            System.out.println(ex.getMessage());
                        } finally { fail(); }

            }
    }
}
