package it.istat.relais.gui.task;

import it.istat.relais.log.TraceLog;
import it.istat.relais.searchSpaceCreation.CrossProduct;
import it.istat.relais.searchSpaceCreation.Blocking;
import it.istat.relais.searchSpaceCreation.SortedNeighborhood;
import it.istat.relais.project.VariablesSetting;
import it.istat.relais.searchSpaceCreation.NestedBlocking;
import it.istat.relais.searchSpaceCreation.ReadingSearchSpace;
import it.istat.relais.searchSpaceCreation.Simhash;
import java.sql.Connection;

public class SearchSpaceCreationTask extends AbstractTask {

    private Connection conn;
    private String reduction;
    private String varblock;
    private String varblock2;
    private int size;
    private VariablesSetting vs;
    private String inputFile;
    private String sep;
    private int rounds;
    private int nGramSize;
    private boolean weigths;
    
    public SearchSpaceCreationTask(Connection conn,VariablesSetting vs) {
        this.conn=conn;
        this.vs=vs;
        reduction = new String("Cross pruduct");
    }
    
    public SearchSpaceCreationTask(Connection conn,VariablesSetting vs,String inpfile,String sep) {
        this.conn=conn;
        this.vs=vs;
        this.inputFile=inpfile;
        this.sep=sep;
        reduction = new String("Read by external");
    }
    
    public SearchSpaceCreationTask(Connection conn,VariablesSetting vs,String varblock) {
        this.conn=conn;
        this.vs=vs;
        this.varblock=varblock;
        reduction = new String("Blocking");
    }
    
    public SearchSpaceCreationTask(Connection conn,VariablesSetting vs,String varblock,boolean union) {
        this.conn=conn;
        this.vs=vs;
        this.varblock=varblock;
        reduction = new String("Blocking Union");
    }
    
    public SearchSpaceCreationTask(Connection conn,VariablesSetting vs,String varblock,int size) {
        this.conn=conn;
        this.vs=vs;
        this.varblock=varblock;
        this.size=size;
        reduction = new String("Sorted neighborhood");
    }
  
    public SearchSpaceCreationTask(Connection conn,VariablesSetting vs,String varblock,String varsort,int size) {
        this.conn=conn;
        this.vs=vs;
        this.varblock=varblock;
        this.varblock2=varsort;
        this.size=size;
        reduction = new String("Nested Blocking");
    }
    
    public SearchSpaceCreationTask(Connection conn,VariablesSetting vs,String varsort,int rounds,int thresh,int grDim,boolean wghts) {
        this.conn=conn;
        this.vs=vs;
        this.varblock=varsort;
        this.size=thresh;
        this.rounds=rounds;
        this.nGramSize=grDim;
        this.weigths=wghts;
        reduction = new String("SimHash");
    }
    
    public SearchSpaceCreationTask(Connection conn,VariablesSetting vs,String varblock,String varsort,int rounds,int thresh,int grDim,boolean wghts) {
        this.conn=conn;
        this.vs=vs;
        this.varblock=varblock;
        this.varblock2=varsort;
        this.size=thresh;
        this.rounds=rounds;
        this.nGramSize=grDim;
        this.weigths=wghts;
        reduction = new String("Blocked SimHash");
    }
        
    public void run() {
        String method;
        TraceLog log = new TraceLog(conn);
        try {
            log.AddMessage("Search Space creation. Reduction: "+reduction,false);
            if (reduction.equals("Sorted neighborhood")) {
                log.AddMessage("Sorting variable: "+varblock+" size: "+size,false);
                SortedNeighborhood snm = new SortedNeighborhood(conn);
                method = snm.CreateSNM(varblock, size);
            } else if (reduction.equals("Blocking")) {
                log.AddMessage("Blocking variable: "+varblock,false);
                Blocking b = new Blocking(conn);
                method = b.CreateBlocks(varblock);
                System.out.println("INFO: Blockmodality table is created");
            } else if (reduction.equals("Blocking Union")) {
                log.AddMessage("Blocking variable: "+varblock,false);
                Blocking b = new Blocking(conn);
                method = b.CreateBlocks(varblock);
                method = reduction;
            }else if (reduction.equals("Nested Blocking")) {
                log.AddMessage("Blocking variable: "+varblock,false);
                log.AddMessage("Sorting variable: "+varblock2+" size: "+size,false);
                NestedBlocking b = new NestedBlocking(conn);
                method = b.CreateSSR(varblock, varblock2, size);
                System.out.println("INFO: Blockmodality table is created");
            } else if (reduction.equals("Read by external")) {
                method = reduction;
                log.AddMessage("Input file: "+varblock,false);
                log.AddMessage("Separator: "+varblock,false);
                ReadingSearchSpace sp = new ReadingSearchSpace(conn);
                method = sp.LoadInput(inputFile, sep);
                
            } else if (reduction.equals("SimHash")) {
                method = reduction;
                log.AddMessage("Hash variables: "+varblock,false);
                Simhash sh = new Simhash(conn,size,rounds,nGramSize,weigths);
                method = sh.CreateSSR(varblock);
                
            } else if (reduction.equals("Blocked SimHash")) {
                method = reduction;
                log.AddMessage("Hash variables: "+varblock,false);
                Simhash sh = new Simhash(conn,size,rounds,nGramSize,weigths);
                method = sh.CreateSSRBlock(varblock2,varblock);
                
            }else {
                CrossProduct cp = new CrossProduct(conn);
                method = cp.CreateCrossProduct();
            }

            vs.UpdateMethod(method);
            log.AddMessage("Search Space created",false);
            succeed();
        } catch (Exception ex) {
            try {
                log.AddMessage("Exception on search space creation","E",false);
                log.AddMessage(ex.getMessage(),"E",false);
            } catch (Exception ex2) { 
                System.out.println(ex2.getMessage());
                System.out.println(ex.getMessage());
            } finally {
                fail();
            }
        }
    }
}
