package it.istat.relais.linkageOtoO;

import it.istat.relais.constants.TablesName;
import java.io.FileWriter;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;

public class HungarianAlgorithm {
    
    private Connection conn;
    private int dimDsa;
    private int dimDsb;
    private int dimTable;
    private double[][] table;
    private boolean[][] tableSigned;
    private ArrayList<String> mvNames;
    private int numLinee = 0;
    private int numZeroMax;
    private int[] numZeroRiga; 
    private int[] numZeroCol; 
    private double[][] initialTable;
    
    
    public HungarianAlgorithm(Connection con)throws Exception {
        this.conn = con;
        this.mvNames = new ArrayList<String>();
        try{
            Statement stm1 =(Statement)conn.createStatement();
            ResultSet rs1 = stm1.executeQuery("SELECT COUNT(*) FROM "+TablesName.nameDsa()+";");
            rs1.next();
            this.dimDsa = rs1.getInt(1);
            stm1.close();
            Statement stm2 =(Statement)conn.createStatement();
            ResultSet rs2 = stm2.executeQuery("SELECT COUNT(*) FROM "+TablesName.nameDsb()+";");
            rs2.next();
            this.dimDsb = rs2.getInt(1);
            stm2.close();
            Statement stm3 = (Statement)conn.createStatement();
            ResultSet rs3 = stm3.executeQuery("SELECT VARIABLE_NAME FROM "+TablesName.nameMatchingVeriables()+";");
            while(rs3.next()){
                this.mvNames.add(rs3.getString(1));
            }
        }catch(Exception e){
            System.out.println(e);
        }
       if(this.dimDsa >= this.dimDsb){
           this.dimTable = this.dimDsa;
       } else{
          this.dimTable = this.dimDsb;  
       }
        
       this.table = new double[this.dimTable][this.dimTable];
       this.tableSigned = new boolean[this.dimTable][this.dimTable];
       this.initialTable = new double[this.dimTable][this.dimTable];
       for(int i=0; i< this.dimTable; i++)
           for(int j=0; j <this.dimTable; j++){
               this.table[i][j]=0;
               this.initialTable[i][j]=0;
           }
    }
    
    public void initializeTable(){
        try{
            String str = "SELECT b.KEY_DSA, b.KEY_DSB, a.P_POST FROM "+TablesName.nameMU()+" a, ";
            str += TablesName.nameSearchSpaceFinal()+" b WHERE concat(";
            for(int i=0; i < this.mvNames.size()-1; i++){
                str += "a."+this.mvNames.get(i)+", ";
            }
            str += "a."+this.mvNames.get(this.mvNames.size()-1)+") = b.PATTERN;";
   
            Statement stm1 = (Statement)conn.createStatement();
            ResultSet rs1 = stm1.executeQuery(str);
            while(rs1.next()){
              table[rs1.getInt(1)-1][rs1.getInt(2)-1] = -rs1.getDouble(3);           
            }
              
        }catch(Exception e){
            System.out.println(e);
        }
        System.out.println("Inizialization done");
         }
    
    public void executeAlgorithm(){
         
        copyInitialData();
         
        if(dimDsa >= dimDsb){
            dimTable = dimDsa;
        } else{
            dimTable = dimDsb;  
        }
        
        table = new double[dimTable][dimTable];
        tableSigned = new boolean[dimTable][dimTable];
        for(int i=0; i< dimTable; i++)
            for(int j=0; j <dimTable; j++)
                table[i][j]=0;
    
        /*STEP 1*/
        double[] minRow = new double[dimTable];
        double[] minCol = new double[dimTable];
        numZeroRiga = new int[dimTable];
        numZeroCol = new int[dimTable];
        int numZeroMaxRiga;
        int numZeroMaxCol;
        double minNumTable;
            
        for(int ii= 0;ii < dimTable; ii++){
                minRow[ii]=table[ii][0];
        }
        
        for(int i =0; i < dimTable; i++){
            for(int j= 0; j< dimTable; j++){
                if(table[i][j] < minRow[i])
                   minRow[i] = table[i][j];
            }
        }
        
        for(int i =0; i < dimTable; i++){
            for(int j= 0; j< dimTable; j++){
                if(table[i][j]!= 0)
                    table[i][j] = table[i][j] - minRow[i];
            }
        }
         for(int ii= 0;ii < dimTable; ii++){
               minCol[ii]=table[0][ii];
         }
        
        for(int j =0; j < dimTable; j++){
            if(!notExistZeroInCol(j)){
                for(int i= 0; i< dimTable; i++){
                    if((table[i][j] < minCol[j]))
                     minCol[j] = table[i][j];
                    }
            }else minCol[j]= 0;
        }
        
        for(int i =0; i < dimTable; i++){
            for(int j= 0; j< dimTable; j++){
                if(table[i][j]!= 0)
                   table[i][j] = table[i][j] - minCol[j];
            }
        }
        do {
            /*STEP 2*/
            clearTableSigned();
            numLinee = 0;
            for(int i=0;i<dimTable;i++){
                numZeroRiga[i]=0;
                numZeroCol[i]=0;
            }
        
            for(int i=0; i<dimTable; i++)
                for(int j=0; j<dimTable; j++){
                    if(table[i][j]== 0){
                        numZeroRiga[i]++;
                        numZeroCol[j]++;
                    }
                }
            
            numZeroMaxRiga = 0;
            numZeroMaxCol = 0;
        
            for(int i=0; i < dimTable; i++){
                if(numZeroRiga[i] > numZeroMaxRiga)
                    numZeroMaxRiga = numZeroRiga[i];
                if(numZeroCol[i] > numZeroMaxCol)
                    numZeroMaxCol = numZeroCol[i];
            }
            
            if(numZeroMaxRiga >= numZeroMaxCol)
                numZeroMax = numZeroMaxRiga;
            else
                numZeroMax = numZeroMaxCol;
            
            for(int k=numZeroMax; k>0; k--){
              for(int i=0; i <dimTable; i++){
                if(numZeroRiga[i]==k && !allZeroRigaSigned(i)){
                    signAllRiga(i);
                    numLinee++;
                }
            }
            for(int j=0; j<dimTable; j++){
                if(numZeroCol[j]== k && !allZeroColSigned(j)){
                    signAllCol(j);
                    numLinee++;
                }       
            }
        }
           
       if((numLinee != dimTable)&&(numLinee < dimTable)){
            
        /*STEP 3*/
            /*STEP 4*/
            minNumTable = findMinNotSigned(); 
            
            for(int i=0; i<dimTable;i++)
                for(int j=0;j<dimTable;j++)
                    if((table[i][j]!=0)){
                        if(tableSigned[i][j]&& allRigaSigned(i) && allColSigned(j)){
                            table[i][j] += minNumTable;
                        }
                        else if(!tableSigned[i][j])  
                            table[i][j] -= minNumTable;
                    }     
            }
           
     }while(numLinee!=dimTable && dimTable>numLinee);
    try{
    FileWriter streem0 = new FileWriter("appo0.txt",false); 
    for(int i=0; i<dimTable;i++){
       for(int j=0; j<dimTable;j++){
          streem0.write(" "+table[i][j]);
       }
       streem0.write("\n");
      }
    streem0.close();
    
    FileWriter streem1 = new FileWriter("appo1.txt",false); 
    for(int i=0; i<dimTable;i++){
       for(int j=0; j<dimTable;j++){
          streem1.write(" "+initialTable[i][j]);
       }
       streem1.write("\n");
      }
    streem1.close();
    
    }catch(Exception e){
       System.out.println(e);
    }
   
  
    }
    
    private void copyInitialData(){
        for(int i=0; i<dimTable;i++)
            for(int j=0;j<dimTable;j++)
                initialTable[i][j]= -table[i][j];
    } 
    
     private void signAllRiga(int i){
         for(int j=0; j<dimTable; j++){
             tableSigned[i][j]=true;
         }
     }
     
     private void signAllCol(int j){
         for(int i=0; i<dimTable; i++){
             tableSigned[i][j]=true;
         }
     }
     
     private boolean allZeroRigaSigned(int i){
         boolean all = true;
         for(int j=0; j<dimTable; j++){
             if(table[i][j]==0)
                 all = all && tableSigned[i][j];
         }
         return all;
     }
     
     private boolean allRigaSigned(int i){
         boolean all = true;
         for(int j=0; j<dimTable; j++){
             all = all && tableSigned[i][j];
         }
         return all;
     }
     private boolean allZeroColSigned(int j){
         boolean all = true;
         for(int i=0; i<dimTable; i++){
             if(table[i][j]==0)
                 all = all && tableSigned[i][j];
         }
         return all;
     }
     
     private boolean allColSigned(int j){
         boolean all = true;
         for(int i=0; i<dimTable; i++){
             all = all && tableSigned[i][j];
         }
         return all;
     }
     private double findMinNotSigned(){
         double min = table[1][1];
         boolean entered_once = false;
         for(int i=0; i<dimTable;i++)
            for(int j=0; j<dimTable;j++){
               if(!tableSigned[i][j] && table[i][j]!= 0) {
                   entered_once = true;
                   if(min>table[i][j])
                       min = table[i][j];
               }
            } 
         if (entered_once)
            return min;
         else
             return 0;
     }
    private boolean notExistZeroInCol(int j){
        boolean exist = false;
        for(int i=0; i<dimTable; i++){
            if(table[i][j]== 0)
                exist =true;
        }
        return exist;
    }
  
    private void clearTableSigned(){
    for(int i=0; i< dimTable; i++)
        for(int j=0; j <dimTable; j++){
         tableSigned[i][j] = false;
    }
    }   
     
    private boolean existOneSignedInCol(int j){
        boolean signed =false;
        for(int i=0;i<dimTable;i++){
            if(tableSigned[i][j]==true && !signed)
                signed = true;
        }
        return signed;
    }
    
     public void writeResult(){
         clearTableSigned();
         try{
       Statement stm = (Statement)conn.createStatement();
        ResultSet rs =stm.executeQuery("SELECT * FROM information_schema.TABLES where table_schema = 'relais' and table_name = 'relais_link_one_to_one';");
        if(rs.first()){
            Statement stm2 =(Statement)conn.createStatement();
             stm2.executeUpdate("DROP TABLE relais_link_one_to_one;");
	     stm2.close();
        }
        stm.close();
        Statement stm1 = (Statement)conn.createStatement();
        stm1.executeUpdate("CREATE TABLE relais_link_one_to_one (I INTEGER, J INTEGER, PATTERN VARCHAR(150), R DOUBLE, P_POST DOUBLE);");
	stm1.close();
         String str;
        boolean notDone;
         for(int i=0; i<this.dimTable; i++){
            notDone = true;
             for(int j=0; j<this.dimTable && notDone; j++){
                  if(this.table[i][j]== 0 && !existOneSignedInCol(j)){
                      notDone = false;
                      tableSigned[i][j]=true;
                      Statement stm2 = (Statement)conn.createStatement();
                      str = "INSERT INTO relais_link_one_to_one VALUES("+ i+", "+j+", null, 0,"+initialTable[i][j]+");";
                      stm2.executeUpdate(str);
                     stm2.close();
                 }
             }
         }
     }catch (Exception e){
         System.out.println(e);
     }
     }            

}
