package it.istat.relais.linkageOtoO;


import it.istat.relais.constants.TablesName;
import it.istat.relais.matchingVariables.MatchingVariables;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
/* Invokes R program for one-to-one matching resolving an LP problem*/
public class LinkageOtoO {
    private Connection connection;
    private String method;
    private String model;
    private MatchingVariables mv;
    private String paramOtoO = "paramOtoO.R";
    
	public LinkageOtoO(Connection con) throws Exception {
		this.connection = con;
		try{
		Statement stm2 = (Statement)connection.createStatement();
		ResultSet rs2 = stm2.executeQuery("SELECT METHOD,MODEL FROM "+TablesName.nameSetting()+";");
		rs2.next();
		this.method = rs2.getString(1);
                this.model = rs2.getString(2);
		stm2.close();
		this.mv = new MatchingVariables(con);
		}catch(Exception e){
			throw e;
		}
	}
	
        /* reduction 1:1 using optimal solution
         * operations of the module:
         * - creates table input with pilote variable
         * - calls method for reduction 1:1 using solution of linear problem
         */
	public void ReduceOtoO() throws Exception {
     	String tableName = null;
        String tableMU = "";
        String lastColumn = "P_POST";
        String valutationValue = "MU.R";
        String ruleVerify = "";
     	FileWriter writer = null;
        
		try{
			tableName = TablesName.nameSearchSpaceFinal();
                        
                        if (this.model.equals("Deterministic")) {
                            tableMU=TablesName.nameMURule();
                            lastColumn="RULE";
                            valutationValue="MU.WEIGHT";
                            ruleVerify=" AND MU.RULE=1";
                        } else {
                            tableMU=TablesName.nameMU();
                        }
			writer = new FileWriter(new File(this.paramOtoO));
			writer.write("inputOtoOTableName = '"+TablesName.nameInputOtoO()+"'\n");
			writer.write("LinkOtoOTableName = '"+TablesName.nameOtoO()+"'\n");
			try{
				Statement stm2 =(Statement)connection.createStatement();
				stm2.executeUpdate("DROP TABLE "+TablesName.nameInputOtoO()+";");
				stm2.close();
				Statement stm3 =(Statement)connection.createStatement();
				stm3.executeUpdate("DROP TABLE "+TablesName.nameOtoO()+";");
				stm3.close();
			}catch(Exception e){
				;
			}
			Statement stm1 = (Statement)connection.createStatement();
			stm1.executeUpdate("CREATE TABLE "+TablesName.nameInputOtoO()+" (I INTEGER, J INTEGER, PATTERN VARCHAR(150), R DOUBLE, "+lastColumn+" DOUBLE);");
			stm1.close();
			
			String matchingvariable="";

                        String[] cols=mv.getContingencyColumns();
			for(int i=0; i<cols.length; i++)
				if (i<(cols.length-1))
					matchingvariable = matchingvariable+cols[i]+", ";
				else
					matchingvariable = matchingvariable+cols[i];
			
			String insertString = "INSERT INTO "+TablesName.nameInputOtoO()+" (I,J,PATTERN,R,"+lastColumn+") SELECT C.KEY_DSA, C.KEY_DSB, C.PATTERN, "+valutationValue+", MU."+lastColumn+
            " FROM  "+tableName.toUpperCase()+" C, "+tableMU+" MU WHERE C.PATTERN = CONCAT("+matchingvariable+")"+ruleVerify;
			Statement stmCreate = (Statement)connection.createStatement();
		    stmCreate.executeUpdate(insertString);
		    stmCreate.close();
		    
		    writer.close(); 
		    LinkageOtoOMethod();
		  
		} catch (Exception e) {
			throw e;
		}
}
        
        /* launch of R code  for reduction 1:1 by solution of linear problem
         * - writes parameters file with name of input table
         * - execution of R code LP.R
          */
	private void LinkageOtoOMethod() throws Exception {
//	  launching LP.R
	   try {
		Runtime rt=Runtime.getRuntime();
	        System.out.println("Computing One to One Linkage...");
	        String RPathname = "LP.R";
	        String path=new File(RPathname).getAbsolutePath(); 
                String str="R CMD BATCH "+path+" ";
                String os = System.getProperty("os.name").toLowerCase();
                if (os.indexOf("win") >= 0){
                    str="R CMD BATCH --max-mem-size=3000M \""+path+"\"";
                }
	        String newStr = str.replace("\\","\\\\");
	        System.out.println("Invoking R...");
	        try {
	        	Process myProc;
			myProc = rt.exec(newStr);
			myProc.waitFor();
                        if(myProc.exitValue()!= 0){
		           	System.out.println("R execution failed");
		           	System.out.println("Reduction to 1:1 matching task failed");
		        }
		        else {
		            Statement stm = (Statement)connection.createStatement();
                             ResultSet rs =stm.executeQuery("SELECT COUNT(*) FROM "+TablesName.nameOtoO()+";");
                            rs.next();
                            if(rs.getInt(1)==0){
                                Statement stm2 =(Statement)connection.createStatement();
				stm2.executeUpdate("DROP TABLE "+TablesName.nameOtoO()+";");
				stm2.close();
                                Exception ex = new Exception("Reduction to 1:1 matching method failed for this data");
                                throw ex;
                         }
                         stm.close();
                         System.out.println("Reduction to 1:1 matching task succeeded");
		         } 
		        
			} catch (IOException e) {
				throw e;
			}
               
	        } catch (InterruptedException e) {
				throw e;
			} 


	}
	
}

