package it.istat.relais.linkageOtoO;

import it.istat.relais.constants.TablesName;
import it.istat.relais.log.TraceLog;
import it.istat.relais.matchingVariables.MatchingVariables;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
/* Invokes R program for one-to-one matching resolving an LP problem after a blocking step*/
public class LinkageOtoOBlocking {
	    private Connection connection;
	    private int numBlockModality;
            private ArrayList<String> blockModality;
            private String blockingKey;
            private String model;
            private String paramOtoO = "paramOtoO.R";
            private MatchingVariables mv;
            int blockOk;
            TraceLog log;
	    
public LinkageOtoOBlocking(Connection con) throws Exception {
        this.connection = con;
        try{
        log = new TraceLog(connection);
        this.mv = new MatchingVariables(con);
        Statement stm2 =(Statement)connection.createStatement();
        ResultSet rs2 = stm2.executeQuery("SELECT BLOCKVAR, MODEL FROM "+TablesName.nameSetting()+";");
        rs2.next();
        this.blockingKey = rs2.getString(1);
        this.model = rs2.getString(2);
        stm2.close();

        this.blockModality = new ArrayList<String>();
        Statement stm3 =(Statement)connection.createStatement();
        ResultSet rs3 = stm3.executeQuery("SELECT * FROM "+TablesName.nameBlockModality()+";");
        while(rs3.next())
                this.blockModality.add(rs3.getString(1));
        stm3.close();
        this.numBlockModality =this.blockModality.size();
        blockOk=0;
        
        }catch(Exception e){
                throw e;
        }
}
	
/* reduction 1:1 using optimal solution
 * operations of the module:
 * - created table input named using blocking key and block number with pilote variable
 * - calld method for reduction 1:1 using solution of linear problem
 */
public void ReduceOtoO() throws Exception {
        String tableName = TablesName.nameSearchSpaceFinal();
        String tableMU = "";
        String lastColumn = "P_POST";
        String valutationValue = "MU.R";
        String ruleVerify = "";
        FileWriter writer = null;
        
        for(int j=0; j<numBlockModality;j++){
            try{
                if (this.model.equals("Deterministic")) {
                    tableMU=TablesName.nameMURule();
                    lastColumn="RULE";
                    valutationValue="MU.WEIGHT";
                    ruleVerify=" AND MU.RULE=1";
                } else {
                    tableMU=TablesName.nameMU()+"_"+this.blockModality.get(j).toUpperCase();
                }

            	Statement stm0 =(Statement)connection.createStatement();
            	ResultSet rs0 = stm0.executeQuery("SELECT TABLE_NAME FROM information_schema.TABLES WHERE upper(TABLE_NAME) = '"+tableMU+"' AND TABLE_SCHEMA = 'relais'");
            	if(rs0.next()){
            	writer = new FileWriter(new File(this.paramOtoO));
				writer.write("inputOtoOTableName = '"+TablesName.nameInputOtoO()+"_"+this.blockingKey.replace(",","_").toUpperCase()+"_"+this.blockModality.get(j).toUpperCase()+"'\n");
				writer.write("LinkOtoOTableName = '"+TablesName.nameOtoO()+"_"+this.blockingKey.replace(",","_").toUpperCase()+"_"+this.blockModality.get(j).toUpperCase()+"'\n");
				try{
					Statement stm2 =(Statement)connection.createStatement();
					stm2.executeUpdate("DROP TABLE "+TablesName.nameInputOtoO()+"_"+this.blockingKey.replace(",","_").toUpperCase()+"_"+this.blockModality.get(j).toUpperCase());
					stm2.close();
					Statement stm3 =(Statement)connection.createStatement();
					stm3.executeUpdate("DROP TABLE "+TablesName.nameOtoO()+"_"+this.blockingKey.replace(",","_").toUpperCase()+"_"+this.blockModality.get(j).toUpperCase());
					stm3.close();
					
				}catch(Exception e){
				}
				Statement stm1 = (Statement)connection.createStatement();
				stm1.executeUpdate("CREATE TABLE "+TablesName.nameInputOtoO()+"_"+this.blockingKey.replace(",","_").toUpperCase()+"_"+this.blockModality.get(j).toUpperCase()+" (I INTEGER, J INTEGER, PATTERN VARCHAR(150), R DOUBLE, "+lastColumn+" DOUBLE);");
				stm1.close();
				
				String matchingvariable="";

				String[] cols=mv.getContingencyColumns();
                                for(int i=0; i<cols.length; i++)
                                        if (i<(cols.length-1))
                                                matchingvariable = matchingvariable+cols[i]+", ";
                                        else
                                                matchingvariable = matchingvariable+cols[i];
				
				String insertString = "INSERT INTO "+TablesName.nameInputOtoO()+"_"+this.blockingKey.replace(",","_").toUpperCase()+"_"+this.blockModality.get(j).toUpperCase()+" (I,J,PATTERN,R,"+lastColumn+") SELECT C.KEY_DSA, C.KEY_DSB, C.PATTERN, "+valutationValue+", MU."+lastColumn+
	            " FROM  "+tableName.toUpperCase()+" C, "+tableMU+" MU WHERE C.BLOCK_NUMBER = '"+this.blockModality.get(j)+"' AND C.PATTERN = CONCAT("+matchingvariable+") "+ruleVerify;
				Statement stmCreate = (Statement)connection.createStatement();
			    stmCreate.executeUpdate(insertString);
			    stmCreate.close();
			    
			    writer.close();
                            System.out.print("Block "+this.blockModality.get(j).toUpperCase()+": ");
			    LinkageOtoOMethod(this.blockModality.get(j));
			    
            	}
            rs0.close();	
			} catch (Exception e) {
				throw e;
			}
	}
		
	if (blockOk==0) throw new Exception("Reduction to 1:1 matching failed");
		   
}

                /* launch of  R code for reduction 1:1 by solution of linear problem
                 * - writes parameters file with name of input table
                 * - execution of R code LP.R
                 */
		private void LinkageOtoOMethod(String blockModality) throws Exception {
//		  launching LP.R
		   try {
			Runtime rt=Runtime.getRuntime();
		        String RPathname = "LP.R";
		        String path=new File(RPathname).getAbsolutePath(); 
		    	
                        String str="R CMD BATCH "+path+" ";
                        String os = System.getProperty("os.name").toLowerCase();
                        if (os.indexOf("win") >= 0){
                            str="R CMD BATCH --max-mem-size=3000M \""+path+"\"";
                        }
		        String newStr = str.replace("\\","\\\\");
		        System.out.println("invoking R...");
		        try {
		        	Process myProc;
				myProc = rt.exec(newStr);
				myProc.waitFor();
			        if(myProc.exitValue()!= 0){
			          	System.out.println("R execution failed");
			           	System.out.println("Reduction to 1:1 matching task failed");
			        }
			        else {
			             Statement stm = (Statement)connection.createStatement();
                                     ResultSet rs =stm.executeQuery("SELECT COUNT(*) FROM "+TablesName.nameOtoO()+"_"+this.blockingKey.replace(",","_").toUpperCase()+"_"+blockModality+";");
                                     rs.next();
                                     if(rs.getInt(1)==0){
                                        Statement stm2 =(Statement)connection.createStatement();
					stm2.executeUpdate("DROP TABLE "+TablesName.nameOtoO()+"_"+this.blockingKey.replace(",","_").toUpperCase()+"_"+blockModality+";");
					stm2.close();
					
                                        stm.close();
                                        log.AddMessage("Block: "+this.blockingKey.replace(",","_")+"_"+blockModality,false);
                                        log.AddMessage("Reduction to 1:1 matching method failed for this block data");
                                        } else {
                                         stm.close();
                                         System.out.println("Reduction to 1:1 matching task succeeded");
                                         blockOk++;
                                     }
			        }
			        
                            } catch (IOException e) {
                                    e.printStackTrace();
                            }
		        } catch (InterruptedException e) {
					throw e;
				} 


		}
		
	}



