package it.istat.relais.linkageOtoO;

import it.istat.relais.constants.TablesName;
import it.istat.relais.log.TraceLog;
import it.istat.relais.matchingVariables.MatchingVariables;
import it.istat.relais.project.VariablesSetting;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
/* Reduction with greedy approach */
public class ReductionByDB {
    private Connection connection;
    private String method;
    private String model;
    private MatchingVariables mv;
    private TraceLog log;
    final int DIMMAX=100000;
    VariablesSetting vs;
    
	public ReductionByDB(Connection con) throws Exception {
		this.connection = con;
		try{
                    this.vs = new VariablesSetting(con);
		    this.method = vs.ReadMethod();
                    this.model = vs.ReadModel();
                    
                    this.mv = new MatchingVariables(con);
                    this.log=new TraceLog(con);

		}catch(Exception e){
			throw e;
		}
	}
	
       /* returns the column name for pilote variable, that is P_POST 
         in the probabilistic approach and RULE in the deterministic one*/
        private String PiloteVariable() {
            if (model.equals("Deterministic"))
                return ("RULE");
            return ("P_POST");
        }
        
        /* creates input table for reduction */
        private void CreateTableInput(String suffix) throws Exception {
        try{
                Statement stmD =(Statement)connection.createStatement();
                stmD.executeUpdate("DROP TABLE "+TablesName.nameInputOtoO()+suffix+";");
                stmD.close();
            }catch(Exception e){
            }
            Statement stmC = (Statement)connection.createStatement();
            stmC.executeUpdate("CREATE TABLE "+TablesName.nameInputOtoO()+suffix+" (I INTEGER UNSIGNED, J INTEGER UNSIGNED, PATTERN VARCHAR(150), R DOUBLE, "+PiloteVariable()+" DOUBLE, ROWNUMBER BIGINT UNSIGNED NOT NULL AUTO_INCREMENT, PRIMARY KEY (ROWNUMBER)) ENGINE=MYISAM;");
            stmC.close();
        }
        
        /* creates table for pairs reduced to 1:1 */
        private void CreateTableLink(String suffix) throws Exception {
        try{
                Statement stmD =(Statement)connection.createStatement();
                stmD.executeUpdate("DROP TABLE "+TablesName.nameOtoO()+suffix+";");
                stmD.close();
            }catch(Exception e){
            }
            Statement stmC = (Statement)connection.createStatement();
            stmC.executeUpdate("CREATE TABLE "+TablesName.nameOtoO()+suffix+" (I INTEGER UNSIGNED, J INTEGER UNSIGNED, PATTERN VARCHAR(150), R DOUBLE, "+PiloteVariable()+" DOUBLE) ENGINE=MYISAM;");
            stmC.close();
            
            Statement stmCInd1 = (Statement)connection.createStatement();
            stmCInd1.executeUpdate("CREATE UNIQUE INDEX IX_LINK_I on "+TablesName.nameOtoO()+suffix+"(I);");
            stmCInd1.close();
            Statement stmCInd2 = (Statement)connection.createStatement();
            stmCInd2.executeUpdate("CREATE UNIQUE INDEX IX_LINK_J on "+TablesName.nameOtoO()+suffix+"(J);");
            stmCInd2.close();
        }
        
        /* reduction to 1:1 using greedy method:
         * - loads pairs of search space in input table ordered by pilote variable
         * - reads input table and inserst each pair in final table if 1:1 constraints are verified
         * - only pairs that verify the rule or have r value > 1 are loaded in input table
         */
	public void ReduceOtoO() throws Exception {
            String tableName;
            int rowloaded=0;
                    
            tableName = TablesName.nameSearchSpaceFinal();

            CreateTableInput("");
            String matchingvariable="";

            String[] cols=mv.getContingencyColumns();
            for(int i=0; i<cols.length; i++)
                    if (i<(cols.length-1))
                            matchingvariable = matchingvariable+cols[i]+", ";
                    else
                            matchingvariable = matchingvariable+cols[i];

            String insertString = "INSERT INTO "+TablesName.nameInputOtoO()+" (I,J,PATTERN,R,"+PiloteVariable()+") ";
            if (model.equals("Deterministic")) {
                insertString = insertString+"SELECT C.KEY_DSA, C.KEY_DSB, C.PATTERN, M.WEIGHT, M.RULE"+
                 " FROM  "+tableName+" C, "+TablesName.nameMURule()+" M WHERE C.PATTERN = CONCAT("+matchingvariable+") AND M.RULE=1 "+
                 " ORDER BY M.WEIGHT DESC;";
            } else {
                insertString = insertString+"SELECT C.KEY_DSA, C.KEY_DSB, C.PATTERN, M.R, M.P_POST"+
                 " FROM  "+tableName+" C, "+TablesName.nameMU()+" M WHERE C.PATTERN = CONCAT("+matchingvariable+") AND R>1 "+
                 " ORDER BY M.R DESC;";
            }
            Statement stmCreate = (Statement)connection.createStatement();
            stmCreate.executeUpdate(insertString);
            stmCreate.close();
            
            String selectCount = "SELECT MAX(ROWNUMBER) FROM "+TablesName.nameInputOtoO()+";";
            Statement stmCount = (Statement)connection.createStatement();			
            ResultSet rsCount = stmCount.executeQuery(selectCount);
            rsCount.next();
            int rowtoload =  rsCount.getInt(1);
            rsCount.close();
            stmCount.close();

            CreateTableLink("");
            
            String selString = "SELECT I,J,PATTERN,R,"+PiloteVariable()+" FROM "+TablesName.nameInputOtoO()+" WHERE 0=0 ";

            PreparedStatement prepInsert;
            Statement stm;
            String insertString1 = "INSERT INTO "+TablesName.nameOtoO()+ " VALUES (?,?,?,?,?);";
            prepInsert = connection.prepareStatement(insertString1);
            
            while (rowloaded < rowtoload) {
                    String selStringDM = selString;
                    if (rowloaded>0) 
                            selStringDM = selStringDM + " AND ROWNUMBER > "+ rowloaded;
                    if (rowtoload>(rowloaded+DIMMAX))
                            selStringDM = selStringDM + " AND ROWNUMBER <= "+ (rowloaded+DIMMAX);
                    selStringDM = selStringDM + " ORDER BY ROWNUMBER;";
                    
                    stm= (Statement)connection.createStatement();	
                    ResultSet rsSelect = stm.executeQuery(selStringDM);
				
                    while (rsSelect.next()) {
                        try {
                            prepInsert.setInt(1,rsSelect.getInt(1));
                            prepInsert.setInt(2,rsSelect.getInt(2));
                            prepInsert.setString(3,rsSelect.getString(3));
                            prepInsert.setDouble(4,rsSelect.getDouble(4));
                            prepInsert.setDouble(5,rsSelect.getDouble(5));
                            prepInsert.executeUpdate();
                        } catch (SQLException mye) {
                            if (!mye.getMessage().startsWith("Duplicate entry"))  throw mye;
                        } catch (Exception e) {
                            throw e;
                        }
                    }
                    
                    rowloaded = rowloaded + DIMMAX;
            }
            
}

        /* reduction to 1:1 using greedy method in case of search space reduced by blocking:
         * - loads pairs of search space in input table ordered by pilote variable
         * - reads input table and inserst each pair in final table if 1:1 constraints are verified
         * - only pairs that verify the rule or have r value > 1 are loaded in input table
         */
	public void ReduceOtoOBlock() throws Exception {
            String tableName;
            int rowloaded;
            int rowtoload;
            String suffix="";
            String suffixmu="";
            ArrayList<String> blockModality;
            Statement stm0;
            
            tableName = TablesName.nameSearchSpaceFinal();
            
            /* create index to access to single block */
            try {
	       String createString = "Create index IX"+tableName+"BLOCK on "+tableName+" (BLOCK_NUMBER);";
	       Statement stmCreate = (Statement)connection.createStatement();
	       stmCreate.executeUpdate(createString);
	       stmCreate.close();
               log.AddMessage("Index created","I",false);
	    } catch (Exception ex){
               if (ex.getMessage().startsWith("Duplicate key name"))
                  log.AddMessage("Index already exists","I",false);
               else
	          throw ex;
	    }
            
            String blockingKey = this.vs.ReadBlockVar();
            blockModality = new ArrayList<String>();
            Statement stm = (Statement) connection.createStatement();
            ResultSet rsbm = stm.executeQuery("SELECT * FROM "+TablesName.nameBlockModality()+";");
            while(rsbm.next())
                    blockModality.add(rsbm.getString(1));
            rsbm.close();
            stm.close();
            
            String matchingvariable="";
            String[] cols=mv.getContingencyColumns();
            for(int i=0; i<cols.length; i++)
                    if (i<(cols.length-1))
                            matchingvariable = matchingvariable+cols[i]+", ";
                    else
                            matchingvariable = matchingvariable+cols[i];
            
            for (int nBlock=0;nBlock<blockModality.size();nBlock++) {
                suffix="_"+blockingKey.replace(",","_").toUpperCase()+"_"+blockModality.get(nBlock).toUpperCase();
                suffixmu="_"+blockModality.get(nBlock).toUpperCase();
                stm0 = (Statement)connection.createStatement();
                ResultSet rs0 = stm0.executeQuery("SELECT TABLE_NAME FROM information_schema.TABLES WHERE upper(TABLE_NAME) = '"+TablesName.nameMU()+suffixmu+"' AND TABLE_SCHEMA = 'relais'");
            	if(!rs0.next() && !model.equals("Deterministic")){
                        rs0.close();
            		continue;
                        
            	}
                rs0.close();
                CreateTableInput(suffix);

                String insertString = "INSERT INTO "+TablesName.nameInputOtoO()+suffix+" (I,J,PATTERN,R,"+PiloteVariable()+") ";
                if (model.equals("Deterministic")) {
                    insertString = insertString+"SELECT C.KEY_DSA, C.KEY_DSB, C.PATTERN, M.WEIGHT, M.RULE"+
                     " FROM  "+tableName+" C, "+TablesName.nameMURule()+" M WHERE C.PATTERN = CONCAT("+matchingvariable+") AND C.BLOCK_NUMBER='"+blockModality.get(nBlock)+"' AND M.RULE=1 "+
                     " ORDER BY M.WEIGHT DESC;";
                } else {
                    insertString = insertString+"SELECT C.KEY_DSA, C.KEY_DSB, C.PATTERN, M.R, M.P_POST"+
                     " FROM  "+tableName+" C, "+TablesName.nameMU()+suffixmu+" M WHERE C.PATTERN = CONCAT("+matchingvariable+") AND C.BLOCK_NUMBER='"+blockModality.get(nBlock)+"' AND M.R>1 "+
                     " ORDER BY M.R DESC;";
                }
                Statement stmCreate = (Statement)connection.createStatement();
                log.AddMessage(insertString,"I",false);
                stmCreate.executeUpdate(insertString);
                stmCreate.close();

                String selectCount = "SELECT MAX(ROWNUMBER) FROM "+TablesName.nameInputOtoO()+suffix+";";
                Statement stmCount = (Statement)connection.createStatement();			
                ResultSet rsCount = stmCount.executeQuery(selectCount);
                rsCount.next();
                rowtoload =  rsCount.getInt(1);
                rowloaded=0;
                rsCount.close();
                stmCount.close();

                CreateTableLink(suffix);

                String selString = "SELECT I,J,PATTERN,R,"+PiloteVariable()+" FROM "+TablesName.nameInputOtoO()+suffix+" WHERE 0=0 ";

                
                String insertString1 = "INSERT INTO "+TablesName.nameOtoO()+suffix+" VALUES (?,?,?,?,?);";
                
                while (rowloaded < rowtoload) {
                PreparedStatement prepInsert;
                prepInsert = connection.prepareStatement(insertString1);
                        String selStringDM = selString;
                        if (rowloaded>0) 
                                selStringDM = selStringDM + " AND ROWNUMBER > "+ rowloaded;
                        if (rowtoload>(rowloaded+DIMMAX))
                                selStringDM = selStringDM + " AND ROWNUMBER <= "+ (rowloaded+DIMMAX);
                        
                      selStringDM = selStringDM + " ORDER BY ROWNUMBER;";
                        Statement stmI= (Statement)connection.createStatement();	
                         ResultSet rsSelect = stmI.executeQuery(selStringDM);
                      
                        while (rsSelect.next()) {
                            try {
                                prepInsert.setInt(1,rsSelect.getInt(1));
                                prepInsert.setInt(2,rsSelect.getInt(2));
                                prepInsert.setString(3,rsSelect.getString(3));
                                prepInsert.setDouble(4,rsSelect.getDouble(4));
                                prepInsert.setDouble(5,rsSelect.getDouble(5));
                                prepInsert.executeUpdate();
                            } catch (SQLException mye) {
                                if (!mye.getMessage().startsWith("Duplicate entry"))  throw mye;
                            } catch (Exception e) {
                                throw e;
                            }
                        }
                      rsSelect.close();
                        stmI.close();
                        prepInsert.close();
                      
                        rowloaded = rowloaded + DIMMAX;
                      }
                System.out.println("Block analysed: "+blockModality.get(nBlock).toUpperCase());
            }
            
}
        
}

