package it.istat.relais.method.read;

import it.istat.relais.datasetInput.*;
import it.istat.relais.constants.TablesName;
import it.istat.relais.log.TraceLog;
import it.istat.relais.method.fs.DeleteRoutFiles;
import it.istat.relais.project.VariablesSetting;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;


public class ReadingMarginals {

	Connection connection;
        TraceLog log;
	
	private final String paramR="param.R";
	private final String FSFailRroot="FSFail";
        private final String FSAllRroot="FSAllert";
        private final String RMessage="RMessage";
	private int varNum;
	private final String RPathname = "mu_from_marginals.R";
        private int numBlockModality;
	private String method;
	private ArrayList<String> blockModality;
	private String blockingKey;
        
        /*constructor of the class*/
	public ReadingMarginals(Connection con) throws Exception {
	    this.connection = con;
            log = new TraceLog(con);
            
            try {
                Statement stm =(Statement)connection.createStatement();
                ResultSet rs;
                rs = stm.executeQuery("SELECT COUNT(*) FROM "+TablesName.nameMatchingVeriables()+";");
                rs.next();
                this.varNum = rs.getInt(1);

		VariablesSetting vs = new VariablesSetting(connection);
		this.method = vs.ReadMethod();
                if (method.endsWith("Blocking")) {
                    Statement stm5 =(Statement)connection.createStatement();
                    ResultSet rs5 = stm5.executeQuery("SELECT BLOCKVAR FROM "+TablesName.nameSetting()+";");
                    rs5.next();
                    this.blockingKey = rs5.getString(1);
                    stm5.close();
		
                    this.blockModality = new ArrayList<String>();
                    Statement stm6 =(Statement)connection.createStatement();
                    ResultSet rs6 = stm6.executeQuery("SELECT * FROM "+TablesName.nameBlockModality()+";");
                    while(rs6.next())
			this.blockModality.add(rs6.getString(1));
                    stm6.close();
                    this.numBlockModality =this.blockModality.size();
                } else {
                    this.numBlockModality = 0;
                    this.blockModality = new ArrayList<String>();
                    this.blockingKey = "";
                }
                
                /* remove old Rout files */
                DeleteRoutFiles drout = new DeleteRoutFiles(FSFailRroot);
                drout.delete();
                drout.changeFilesName(FSAllRroot);
                drout.delete();
                drout.changeFilesName(RMessage);
                drout.delete();
            } catch(Exception e){
                e.printStackTrace();
                throw e;
	    }
	}
	
	/*function that implements the reading of the marginals probabilities
         * of model matching variables.
         * Input:
         * pathFile: pattern of the input file 
         * sep: character separating the columns in the file 
         * If the file track is respected and all variables are represented
         * a table of marginal probabilities is created and populated */

        public void LoadFile(String pathFile, String sep) throws Exception {
		Statement stmDSA = null;
		RecordReader readerDSA;
		Record schemaDSA;
		ArrayList<Marginal> margs; 
		String createDSA;
		int dimDSA;
		String rs;
                Record row;
                int jj;
                Statement stmI;
                
		try{
			readerDSA = new RecordReader(pathFile, sep);
			schemaDSA = new Record(readerDSA.read(), sep);
			dimDSA = schemaDSA.size();
                        if (dimDSA!=5) {
                            log.AddMessage("Error: Input file must have 5 columns","E");
                            throw new Exception("Error il reading file input");
                        }
			
                        //verify the marginals
                        margs = new ArrayList<Marginal>();
                        String sqlsel = "SELECT VARIABLE_NAME FROM "+TablesName.nameMatchingVeriables();
                        Statement stmMV = (Statement)connection.createStatement();
			ResultSet rsMV = stmMV.executeQuery(sqlsel);
			while(rsMV.next()){
				margs.add(new Marginal(rsMV.getString(1),"1"));
				margs.add(new Marginal(rsMV.getString(1),"0"));
			}
			stmMV.close();
                        
                        while((rs=readerDSA.read())!= null){
                           row = new Record(rs,sep);
                           for (Marginal m:margs) 
                               if (m.is(row.get(0).toString(),row.get(1).toString()))
                                   m.update(Double.parseDouble(row.get(2).toString()), Double.parseDouble(row.get(3).toString()),
                                            Double.parseDouble(row.get(4).toString()));
                        } 
                        
                        for (Marginal m:margs) {
                            if (!m.isFound()) {
                                log.AddMessage("Error: marginals for "+m.variable+" is not complete","E");
                                throw new Exception("Error il reading file input");
                            }
                        }
                        
			Statement stm1 = (Statement)connection.createStatement();			
			stm1.executeUpdate("DROP TABLE IF EXISTS "+TablesName.nameMUVariable());
			stm1.close();

			//creation of table marginals_probabilities 
			createDSA = "CREATE TABLE "+TablesName.nameMUVariable()+" (VARIABLE VARCHAR(250), COMPARISON VARCHAR(1), M DOUBLE, U DOUBLE, P DOUBLE, PRIMARY KEY (VARIABLE,COMPARISON));";
			
			stmDSA = (Statement)connection.createStatement();
			stmDSA.executeUpdate(createDSA);
			stmDSA.close();

                        /*String name = "os.name";
                        String os = System.getProperty(name).toLowerCase();
                        
                        if (os.indexOf("win") >= 0)*/
			
                        //population of table
                        stmI = (Statement)connection.createStatement();
			for (Marginal m:margs) {
                            stmI.executeUpdate(m.toInsertString(TablesName.nameMUVariable()));
                        }                           
                        
            	}catch (Exception e) {
                    System.out.println(e.getMessage());
                    throw e;		
		
		}
	}
        
        public void MUEval() throws Exception {
            if (method.endsWith("Blocking")) MUEvalBlock();
            else MUEvalNoBlock();
        }
        
        public void MUEvalNoBlock() throws Exception {

		    FileWriter writer = null;
                    TraceLog log = new TraceLog(connection);
                    String FSFailR=FSFailRroot+".Rout";
                    String FSAllR=FSAllRroot+".Rout";
                    
		    	//Writing the parameters for the R file in a file with extension .R
		    	try 
				  {
					writer = new FileWriter(new File(this.paramR));
					writer.write("nvar="+this.varNum+System.getProperty("line.separator"));
					writer.write("contingencyTableName= '"+TablesName.nameContingency()+"'"+System.getProperty("line.separator"));
					writer.write("muTableName = '"+TablesName.nameMU()+"'"+System.getProperty("line.separator"));
                                        writer.write("varmuTableName = '"+TablesName.nameMUVariable()+"'"+System.getProperty("line.separator"));
					writer.write("percorso_fail=\""+FSFailR+"\""+System.getProperty("line.separator"));
                                        writer.write("percorso_allert=\""+FSAllR+"\""+System.getProperty("line.separator"));
					if(new File(FSFailR).exists())
						new File(FSFailR).delete();
				  }catch (Exception ex) {
					ex.printStackTrace();
                                        throw ex;
				  }
				  finally 
				  {
				   try 
				   { 
					writer.close(); 
				   }catch(Exception e){
					   System.out.println("Error!!");
				   } 
				  }
				  //launching mu_gen_embedded.R
				  try {
                                        Statement stmDrop = (Statement)connection.createStatement();
					stmDrop.executeUpdate("DROP TABLE IF EXISTS "+TablesName.nameMU()+";");
					 	
					Runtime rt=Runtime.getRuntime();
		            log.AddMessage("Evaluation of MU table...");          
		            String path=new File(RPathname).getAbsolutePath();
                            
                            String str="Rscript "+path+" ";
                            String os = System.getProperty("os.name").toLowerCase();
                            if (os.indexOf("win") >= 0){
                                str="RSCRIPT \""+path+"\"";
                            }
                            
		            String newStr = str.replace("\\","\\\\");
                            log.AddMessage("Invoking R...",false);
                
		            Process myProc=rt.exec(newStr);
		          InputStream is = myProc.getInputStream();
                          InputStream errs = myProc.getErrorStream();
                            InputStreamReader isr = new InputStreamReader(is);
                            BufferedReader br = new BufferedReader(isr);
                            InputStreamReader iserr = new InputStreamReader(errs);
                            BufferedReader brerr = new BufferedReader(iserr);
                            FileWriter errwriter = new FileWriter(new File(this.RMessage+".Rout"));
                            
                           String line;
                            while ((line = br.readLine()) != null) {
                                log.AddMessage(line);
                            }
                            while ((line = brerr.readLine()) != null) {
                                errwriter.write(line);
                                errwriter.write(System.getProperty("line.separator"));
                                log.AddMessage(line, false);
                            }
                            errwriter.close();
                            
                            myProc.waitFor();
		            if(myProc.exitValue()!= 0){
		            	Exception fail = new Exception("R execution failed");
                                throw fail;
		            }
		            else {
                                log.AddMessage("Estimation succeeded",false);	
		            }  
		        } catch (Exception ex) {
                            throw ex;
		        } 
	}  
        
        public boolean NotExists(String table) {
            try {
                Statement stm =(Statement)connection.createStatement();
                ResultSet rs = stm.executeQuery("SELECT TABLE_NAME FROM information_schema.TABLES WHERE upper(TABLE_NAME) = '"+table.toUpperCase()+"' AND TABLE_SCHEMA = 'relais'");
            	if(rs.next()) return false;
                return true;
            } catch (Exception ex) {
                System.out.println(ex.getMessage());
                return false;
            }
        }
        public void MUEvalBlock() throws Exception {

		    FileWriter writer = null;
                    TraceLog log = new TraceLog(connection);
                    String FSFailR=FSFailRroot+".Rout";
                    String FSAllR=FSAllRroot+".Rout";
                    String nameMUBlock;
                    String nameCTBlock;
                    
                    for(int i=0; i<numBlockModality;i++) {
		    	//Writing the parameters for the R file in a file with extension .R
                        nameMUBlock = TablesName.nameMU()+"_"+this.blockingKey.replace(",","_").toUpperCase()+"_"+this.blockModality.get(i).toUpperCase();
		    	if (NotExists(nameMUBlock)) {
                            nameCTBlock = TablesName.nameContingency()+"_"+this.blockingKey.replace(",","_").toUpperCase()+"_"+this.blockModality.get(i).toUpperCase();
                            try 
				  {
					writer = new FileWriter(new File(this.paramR));
					writer.write("nvar="+this.varNum+System.getProperty("line.separator"));
					writer.write("contingencyTableName= '"+nameCTBlock+"'"+System.getProperty("line.separator"));
					log.AddMessage("Evaluation of MUTable = '"+nameMUBlock);
                                        writer.write("muTableName = '"+nameMUBlock+"'"+System.getProperty("line.separator"));
                                        writer.write("varmuTableName = '"+TablesName.nameMUVariable()+"'"+System.getProperty("line.separator"));
					writer.write("percorso_fail=\""+FSFailR+"\""+System.getProperty("line.separator"));
                                        writer.write("percorso_allert=\""+FSAllR+"\""+System.getProperty("line.separator"));
					if(new File(FSFailR).exists())
						new File(FSFailR).delete();
				  }catch (Exception ex) {
					ex.printStackTrace();
                                        throw ex;
				  }
				  finally 
				  {
				   try 
				   { 
					writer.close(); 
				   }catch(Exception e){
					   System.out.println("Error!!");
				   } 
				  }
				  //launching mu_gen_embedded.R
				  try {
                                        Statement stmDrop = (Statement)connection.createStatement();
					stmDrop.executeUpdate("DROP TABLE IF EXISTS "+TablesName.nameMU()+";");
					 	
					Runtime rt=Runtime.getRuntime();         
                                String path=new File(RPathname).getAbsolutePath();
                                String str="RSCRIPT \""+path+"\"";
                                String newStr = str.replace("\\","\\\\");
                                log.AddMessage("Invoking R...",false);

                                Process myProc=rt.exec(newStr);
                                InputStream is = myProc.getInputStream();
                                InputStream errs = myProc.getErrorStream();
                                InputStreamReader isr = new InputStreamReader(is);
                                BufferedReader br = new BufferedReader(isr);
                                InputStreamReader iserr = new InputStreamReader(errs);
                                BufferedReader brerr = new BufferedReader(iserr);
                                FileWriter errwriter = new FileWriter(new File(this.RMessage+".Rout"));

                               String line;
                                while ((line = br.readLine()) != null) {
                                    log.AddMessage(line);
                                }
                                while ((line = brerr.readLine()) != null) {
                                    errwriter.write(line);
                                    errwriter.write(System.getProperty("line.separator"));
                                    log.AddMessage(line, false);
                                }
                                errwriter.close();

                                myProc.waitFor();
                                if(myProc.exitValue()!= 0){
                                    Exception fail = new Exception("R execution failed");
                                    throw fail;
                                }
                                else {
                                    log.AddMessage("Estimation succeeded",false);	
                                }  
		            } catch (Exception ex) {
                                throw ex;
		            } 
                        }
                    }
                    
	}  
	
}