package it.istat.relais.preprocessing;

import it.istat.relais.project.VariablesSchema;
import it.istat.relais.constants.TablesName;
import it.istat.relais.datasetInput.Record;
import it.istat.relais.datasetInput.RecordReader;
import it.istat.relais.gui.FieldMerge;
import it.istat.relais.gui.FieldSplit;
import it.istat.relais.gui.FieldStandardization;
import it.istat.relais.gui.ReadConversionFile;
import it.istat.relais.project.VariablesSetting;
import java.io.File;
import java.io.FileWriter;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;

public class StandardizationField {

Connection conn;
VariablesSetting vs;

/*Constructor*/
 public StandardizationField(Connection conn) {
	this.conn=conn;
        vs = new VariablesSetting(conn);
 }

/*Create a new variable with standardizated value of original variable*/
 public void createNewVariable(FieldStandardization df) throws Exception {
     if (vs.ReadProjectType().equalsIgnoreCase("Deduplication")) {
         createNewVariableSet(df,"A");
         createNewVariableSet(df,"B");
     } else {
         createNewVariableSet(df,df.getSet());
     }
 }

 /*Create a new variable with inaccurate converted of original variable*/
 public void convertVariable(ReadConversionFile rcf) throws Exception {
     if (vs.ReadProjectType().equalsIgnoreCase("Deduplication")) {
         convertVariableSet(rcf,"A");
         convertVariableSet(rcf,"B");
     } else {
         convertVariableSet(rcf,rcf.getSet());
     }
 }
 
 public void createMergeVariable(FieldMerge fm) throws Exception {
      if (vs.ReadProjectType().equalsIgnoreCase("Deduplication")) {
         createMergeVariableSet(fm,"A");
         createMergeVariableSet(fm,"B");
     } else {
         createMergeVariableSet(fm,fm.getSet());
     }
 }
 
 public void splitVariable(FieldSplit fsp) throws Exception {
      if (vs.ReadProjectType().equalsIgnoreCase("Deduplication")) {
         splitVariableSet(fsp,"A");
         splitVariableSet(fsp,"B");
     } else {
         splitVariableSet(fsp,fsp.getSet());
     }
 }
 
/*Create a new variable with standardizated value of original variable*/
 private void createNewVariableSet(FieldStandardization df,String set) throws Exception {
	
    String table,standardization;
    
    VariablesSchema vsc= new VariablesSchema(conn);
    vsc.addField(set, df.getNewField());
    
    table=TablesName.nameDsa();
    if (set.equals("B")) table=TablesName.nameDsb();
    
    standardization = df.getField();
    if (df.getRemoveBlanks()) {
        standardization = "REPLACE("+standardization+",' ','')";
    }
    if (df.getRemoveChar()) {
        standardization = "REPLACE("+standardization+",'"+df.getChar().replace("'", "''").replace("\\", "\\\\")+"','')";
    }
    if (df.getRemoveSpecials()) {
        standardization = "REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE("+
                          standardization+
                          ",'.',''),',',''),';',''),':',''),'\\\\',''),'/',''),'''',''),'\"','')";
    }
    if (df.getConvertCase()) {
        standardization = df.getCase().toUpperCase()+"("+standardization+")";
    }
    
    Statement stm = conn.createStatement();
    String sqlPopolateField = "UPDATE "+table+" SET "+df.getNewField()+" = "+standardization;
    
    //System.out.println(sqlPopolateField);
    stm.executeUpdate(sqlPopolateField);
 }

 /*Create a new variable with inaccurate converted of original variable*/
 private void convertVariableSet(ReadConversionFile rcf,String set) throws Exception {
	
    String table;
    String sqlPopolateField;
    String tableConv;
    String sqlUpd;
    
    VariablesSchema vsc= new VariablesSchema(conn);
    vsc.addField(set, rcf.getNewField());
    
    table=TablesName.nameDsa();
    if (set.equals("B")) table=TablesName.nameDsb();
    tableConv=table+"_CONV";
    
    Statement stm = conn.createStatement();
    
    sqlUpd="DROP TABLE IF EXISTS "+tableConv;
    stm.executeUpdate(sqlUpd);
    
    sqlUpd="CREATE TABLE "+tableConv+" (INACCURATE VARCHAR(250), CONVERTED VARCHAR(250))";
    stm.executeUpdate(sqlUpd);
    
    sqlUpd="CREATE UNIQUE INDEX IX"+tableConv+" ON "+tableConv+" (INACCURATE)";
    stm.executeUpdate(sqlUpd);
    
    /* load data from input file to table tableConv */
    String rs;
    int nRow=0;
    Record row;
    RecordReader reader = new RecordReader(rcf.getFile(), rcf.getSep());
    while((rs=reader.read())!= null){
        nRow++;
        row = new Record(rs,rcf.getSep());
        if (row.size()<2) throw new Exception ("Error at line "+nRow+" two field are required: '"+rs+"'");
        sqlUpd = "INSERT INTO "+tableConv+" VALUES ('"+row.get(0).toString().replaceAll("'", "''")+"','"+row.get(1).toString().replaceAll("'", "''")+"')";
        try {
           stm.executeUpdate(sqlUpd);
        } catch (SQLException mye) {
           if (!mye.getMessage().startsWith("Duplicate entry"))  throw mye;
        } catch (Exception e) {
           throw e;
        }
    }
    reader.close();
    
    sqlPopolateField = "UPDATE "+table+" SET "+rcf.getNewField()+" = "+rcf.getField();
    stm.executeUpdate(sqlPopolateField);
    
    sqlPopolateField = "UPDATE "+table+" D SET "+rcf.getNewField()+" = \n"+
                       " (SELECT C.CONVERTED FROM "+tableConv+" C WHERE C.INACCURATE=D."+rcf.getField()+")\n"+
                       " WHERE EXISTS (SELECT 1 FROM "+tableConv+" C WHERE C.INACCURATE=D."+rcf.getField()+")";
    //System.out.println(sqlPopolateField);
    stm.executeUpdate(sqlPopolateField);
 }
 
 /*Create a new variable with merge value of original variables*/
 private void createMergeVariableSet(FieldMerge df,String set) throws Exception {
	
    String table,standardization;
    
    VariablesSchema vsc= new VariablesSchema(conn);
    vsc.addField(set, df.getNewField());
    
    table=TablesName.nameDsa();
    if (set.equals("B")) table=TablesName.nameDsb();
    
    standardization = "CONCAT("+df.getFieldToMerge()+")";

    
    Statement stm = conn.createStatement();
    String sqlPopolateField = "UPDATE "+table+" SET "+df.getNewField()+" = "+standardization;
    
    //System.out.println(sqlPopolateField);
    stm.executeUpdate(sqlPopolateField);
 }

/*Create two new variables by splitting value of original variable*/
 private void splitVariableSet(FieldSplit df,String set) throws Exception {
	
    String table,standardization1="",standardization2="";
    String where="",where2="";
    
    VariablesSchema vsc= new VariablesSchema(conn);
    vsc.addField(set, df.getNewField());
    vsc.addField(set, df.getNewField2());
    
    table=TablesName.nameDsa();
    if (set.equals("B")) table=TablesName.nameDsb();
    
    if (df.getTypeSeparator()) {
        if (df.getStartBegin()) {
            standardization1 = "SUBSTR("+df.getField()+",1,INSTR("+df.getField()+",'"+df.getSeparator()+"')-1)";
            standardization2 = "SUBSTR("+df.getField()+",INSTR("+df.getField()+",'"+df.getSeparator()+"')+1)";
        } else {
            standardization1 = "REVERSE(SUBSTR(REVERSE("+df.getField()+"),1,INSTR(REVERSE("+df.getField()+"),'"+df.getSeparator()+"')-1))";
            standardization2 = "REVERSE(SUBSTR(REVERSE("+df.getField()+"),INSTR(REVERSE("+df.getField()+"),'"+df.getSeparator()+"')+1))";
        }   
    } else {
        if (df.getStartBegin()) {
            standardization1 = "SUBSTR("+df.getField()+",1,"+df.getLength()+")";
            standardization2 = "SUBSTR("+df.getField()+","+df.getLength()+"+1)";
        } else {
            standardization1 = "SUBSTR("+df.getField()+",-"+df.getLength()+")";
            standardization2 = "SUBSTR("+df.getField()+",1,LENGTH("+df.getField()+")-"+df.getLength()+")";
            where = " WHERE LENGTH("+df.getField()+") > "+df.getLength();
            where2 = " WHERE LENGTH("+df.getField()+") <= "+df.getLength();
        }   
    }
    
    Statement stm = conn.createStatement();
    String sqlPopolateField = "UPDATE "+table+" SET "+df.getNewField()+" = "+standardization1+" , "+df.getNewField2()+" = "+standardization2+where;
    
    //System.out.println(sqlPopolateField);
    stm.executeUpdate(sqlPopolateField);
    
    if (!where.equals("")) {
        sqlPopolateField = "UPDATE "+table+" SET "+df.getNewField()+" = "+df.getField()+where2;
        //System.out.println(sqlPopolateField);
        stm.executeUpdate(sqlPopolateField);
    }
    
 }

}