package it.istat.relais.qualityMetaData;

import it.istat.relais.constants.TablesName;
import it.istat.relais.datasetInput.Record;
import it.istat.relais.datasetInput.RecordReader;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;

public class Accuracy {
private Connection conn;
private String DS;
private String varName;
private int dimDS;

public Accuracy(Connection conn, String DS, int dimDS, String varName){
    this.conn = conn;
    this.DS = DS;
    this.dimDS = dimDS;
    this.varName = varName.toUpperCase();
}

/* returns accuracy for a variable
 * operations of the module:
 * - loads dictionary table from input file
 * - evaluation of accuracy using the dictionary table
 */
public double EvaluateAccuracy(String fileName, String separator){
    RecordReader reader;
    String input;
    Record record;
    int numAccurate;
    double accuracy = -1;
     
    try{
     Statement stm0 = (Statement)conn.createStatement();
     ResultSet rs0 = stm0.executeQuery("SELECT TABLE_NAME FROM information_schema.TABLES WHERE TABLE_SCHEMA = 'relais' AND TABLE_NAME = '"+TablesName.nameAccuracyDictionary()+"';");
        if(!rs0.next()){
            createTableAccuracyDictionary();
        }
        deleteTableAccuracyDictionary();
        reader = new RecordReader(fileName, separator);
        record = reader.readRecord();
	while((record = reader.readRecord()) != null){
             for(int i=0; i<record.size(); i++){
        	 input = "INSERT INTO "+TablesName.nameAccuracyDictionary()+" values ('"+varName+"', '";
		 input = input + record.get(i).toString()+"');"; 
		 Statement stm = (Statement)conn.createStatement();
		 stm.executeUpdate(input);
		 stm.close();
            }
	}
			
	String count ="SELECT COUNT(*) FROM "+DS+" WHERE "+varName+" IN ";
	count = count + "(SELECT COLVALUE FROM "+TablesName.nameAccuracyDictionary()+" WHERE COLNAME = '"+ varName +"');";
	
	Statement stm1 = (Statement)conn.createStatement();
	ResultSet rs1 = stm1.executeQuery(count);
	rs1.next();
	numAccurate = rs1.getInt(1);
	rs1.close();
	stm1.close();
        
        accuracy =((double)(numAccurate)/(double)this.dimDS);
			
        }catch(Exception e){
            e.printStackTrace();
	}
	return accuracy;
    }
	
    /* creates table for dictionary of accuracy */
	public void createTableAccuracyDictionary(){
            try{
                Statement stm = (Statement)conn.createStatement();
                stm.executeUpdate("CREATE TABLE "+TablesName.nameAccuracyDictionary() +"(COLNAME VARCHAR(150), COLVALUE VARCHAR(150));");
                stm.close();
            }catch(Exception e){
                e.printStackTrace();
            }
	}
	
    /* drops table for dictionary of accuracy */
	public void dropTableAccuracyDictionary(){
            try{
            	Statement stm = (Statement)conn.createStatement();
		stm.executeUpdate("DROP TABLE "+TablesName.nameAccuracyDictionary()+";");
		stm.close();
            }catch(Exception e){
		;
            }	
        }
        
    /* deletes table of accuracy dictionary for the variable */
        private void deleteTableAccuracyDictionary(){
            try{
            	Statement stm = (Statement)conn.createStatement();
		stm.executeUpdate("DELETE FROM "+TablesName.nameAccuracyDictionary()+" WHERE COLNAME='"+varName+"';");
		stm.close();
            }catch(Exception e){
		;
            }	
        }

}
