package it.istat.relais.qualityMetaData;

import it.istat.relais.constants.TablesName;
import it.istat.relais.datasetInput.Record;
import it.istat.relais.datasetInput.RecordReader;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;

public class Consistency {
private Connection conn;
private String DS;
private String varName;
private int dimDS;

public Consistency(Connection conn, String DS, int dimDS, String varName){
    this.conn=conn;
    this.DS=DS;
    this.dimDS =dimDS;
    this.varName=varName;
}

/* returns consistency for the variable
 * operations of the module:
 * - loads dictionary table from input file
 * - evaluation of consistency using the dictionary table
 */
public double EvaluateConsistency(String fileName, String separator){
    RecordReader reader;
    String input;
    Record record;
    int numConsistency;
    String colName1,colName2;
    double consistency = -1;
		
    createTableConsistencyDictionary();
    try{
	reader = new RecordReader(fileName, separator);
        record = reader.readRecord();
        colName1 = record.get(0).toString().toUpperCase();
        colName2 = record.get(1).toString().toUpperCase();
    if(colName1.equalsIgnoreCase(varName)||colName2.equalsIgnoreCase(varName)){
	while((record = reader.readRecord()) != null){
	 input = "INSERT INTO "+TablesName.nameConsistencyDictionary()+"_"+varName+" values ('"+colName1+"', '"+colName2+"', ";
	 input = input + "'"+record.get(0).toString()+"', '"+record.get(1).toString()+"');"; 
	 Statement stm = (Statement)conn.createStatement();
         stm.executeUpdate(input);
	 stm.close();
	}
			
	String count ="SELECT COUNT(*) FROM "+DS+" WHERE (("+colName1+", "+colName2+") IN ";
	count = count + "(SELECT COLVALUE1, COLVALUE2 FROM "+TablesName.nameConsistencyDictionary()+"_"+varName+" WHERE COLNAME1 = '"+ colName1 +"' AND COLNAME2 = '"+ colName2 +"')";
	count = count +") OR (("+colName1+", "+colName2+") IN ";
	count = count + "(SELECT COLVALUE2, COLVALUE1 FROM "+TablesName.nameConsistencyDictionary()+"_"+varName+" WHERE COLNAME1 = '"+ colName2 +"' AND COLNAME2 = '"+ colName1 +"'));";
			
	Statement stm1 = (Statement)conn.createStatement();
	ResultSet rs1 = stm1.executeQuery(count);
	rs1.next();
	numConsistency = rs1.getInt(1);
	rs1.close();
	stm1.close();
        
        consistency =((double)(numConsistency)/(double)(dimDS));
        }
        else
            System.out.println("Unable to evaluate concistency for variable "+varName+" : the dictionary do not contain the variable selected");
	}catch(Exception e){
            e.printStackTrace();
	}
	return consistency;
    }
	
    /* creates table for dictionary of consistency named using the variable name */ 
	public void createTableConsistencyDictionary(){
            dropTableConsistencyDictionary();
            try{
                Statement stm = (Statement)conn.createStatement();
		stm.executeUpdate("CREATE TABLE "+TablesName.nameConsistencyDictionary()+"_"+varName+" (COLNAME1 VARCHAR(150), COLNAME2 VARCHAR(150), COLVALUE1 VARCHAR(150), COLVALUE2 VARCHAR(150));");
		stm.close();
            }catch(Exception e){
		e.printStackTrace();
            }
	}
	
    /* drops table for dictionary of consistency named using the variable name */ 
	public void dropTableConsistencyDictionary(){
            try{
		Statement stm = (Statement)conn.createStatement();
		stm.executeUpdate("DROP TABLE "+TablesName.nameConsistencyDictionary()+"_"+varName+";");
		stm.close();
            }catch(Exception e){
		;
            }	
	}	
}
