package it.istat.relais.result;


import it.istat.relais.constants.TablesName;
import it.istat.relais.log.TraceLog;
import it.istat.relais.matchingVariables.MatchingVariables;
import it.istat.relais.project.ReconciledSchema;
import it.istat.relais.project.VariablesSetting;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

public class ResultTable {
	private Connection connection;
	private double valThresholdM;
	private double valThresholdU;
	private String searchMethod;
	private String blockVar;
        private String model;
        private String ptype;
        private TraceLog log;
        private MatchingVariables mv;
        private ReconciledSchema rsc;
	
	public ResultTable(Connection conn) throws Exception {

		this.connection = conn;
		try {
                        VariablesSetting vs = new VariablesSetting(conn);

			valThresholdM = vs.ReadThresholdM();
			valThresholdU = vs.ReadThresholdU();
			searchMethod = vs.ReadMethod();
			blockVar = vs.ReadBlockVar();
                        model = vs.ReadModel();
                        ptype = vs.ReadProjectType();
                        
                        mv = new MatchingVariables(conn);
                        log = new TraceLog(conn);
                        rsc = new ReconciledSchema(conn);
                        if (searchMethod == null) searchMethod = " ";
		} catch (Exception ex){
		    throw ex;
		}	

	}

        /* creation of generic result table struct with common attributes of all datasets */
	private void CreateTable(String tableName) throws Exception {
		
	try {
		String[] varA=rsc.getVariableReconciled("A");
                String[] varB=rsc.getVariableReconciled("B");
		String dsaColumns = "A_KEY_DSA INT(10)";
		String dsbColumns = "B_KEY_DSB INT(10)";
		for (int ind=0;ind<varA.length;ind++) {
			dsaColumns = dsaColumns + ", A_"+varA[ind].toUpperCase()+" VARCHAR(250)";
			dsbColumns = dsbColumns + ", B_"+varB[ind].toUpperCase()+" VARCHAR(250)";
		}
		
		try{	
			Statement stmDrop = (Statement)connection.createStatement();
			stmDrop.executeUpdate("DROP table "+tableName);
			stmDrop.close();
			}catch(SQLException e){
				;
			}
		String createString = "Create table "+tableName+" ( "+dsaColumns+","+dsbColumns+OtherColumnsCreate()+",ROWNUMBER BIGINT unsigned not null AUTO_INCREMENT, PRIMARY KEY (ROWNUMBER)) ENGINE=MYISAM;";
		Statement stmCreate = (Statement)connection.createStatement();
	    stmCreate.executeUpdate(createString);
	    stmCreate.close();	
	} catch (Exception ex){
	    throw ex;
	}
	}
        /* creating an index for the detection of blocks in the search space  */
	private void CreateIndexBlock(String tableName) throws Exception {
		
	try {

	    String createString = "Create index IX"+tableName+"BLOCK on "+tableName+" (BLOCK_NUMBER);";
	    Statement stmCreate = (Statement)connection.createStatement();
	    stmCreate.executeUpdate(createString);
	    stmCreate.close();
            log.AddMessage("Index created","I",false);
	} catch (Exception ex){
            if (ex.getMessage().startsWith("Duplicate key name"))
               log.AddMessage("Index already exists","I",false);
            else
	    throw ex;
	}
	}
	
        /* loads record in result table */
	private void AppendTable(String tableName, String tableMUName, String AndConditionMUTable, String dataTableName, String varBlock, String orderKey ) throws Exception {
		try {
			String[] varA=rsc.getVariableReconciled("A");
                        String[] varB=rsc.getVariableReconciled("B");
                        String dsaColumns = "A.KEY_DSA ";
			String dsbColumns = "B.KEY_DSB ";
			String tdsaColumns = "A_KEY_DSA ";
			String tdsbColumns = "B_KEY_DSB ";
			for (int ind=0;ind<varA.length;ind++) {
			       dsaColumns = dsaColumns + ", A."+varA[ind].toUpperCase();
			       dsbColumns = dsbColumns + ", B."+varB[ind].toUpperCase();
                               tdsaColumns = tdsaColumns + ", A_"+varA[ind].toUpperCase();
			       tdsbColumns = tdsbColumns + ", B_"+varB[ind].toUpperCase();
			}
			
			String matchvariable="";
			int numVar=0;
			String[] cols = mv.getContingencyColumns();
			while(numVar<cols.length){
				if (numVar> 0) matchvariable = matchvariable + ", ";
				matchvariable = matchvariable+"MU."+cols[numVar];
  				numVar++;                            
			}
                        
			String insertString = "INSERT INTO "+tableName+" ("+tdsaColumns+","+tdsbColumns+OtherColumnsAppend()+") SELECT "+dsaColumns+","+dsbColumns+OtherColumnsAppendMU()+
			                      " FROM "+TablesName.nameDsa()+" A, "+TablesName.nameDsb()+" B, "+dataTableName+" C, "+tableMUName+" MU WHERE A.KEY_DSA=C.KEY_DSA AND B.KEY_DSB=C.KEY_DSB"+
			                      " AND C.PATTERN = concat("+matchvariable+") "+AndConditionMUTable;

			if (!varBlock.equals("NO_BLOCK"))
				insertString = insertString + " AND C.BLOCK_NUMBER ='"+varBlock+"' ";
			
                        insertString=insertString+" ORDER BY "+ orderKey;
                        
                        log.AddMessage("insert into "+tableName+" from "+tableMUName+" where ..."+insertString.substring(insertString.length()-45),false);
			Statement stmCreate = (Statement)connection.createStatement();
		    stmCreate.executeUpdate(insertString);
		    stmCreate.close();	
		} catch (Exception ex){
		    throw ex;
		}
        }

        /* loads record in match table if model choosen is equality match 
         * Operations of the module:
         * - creation of an index in dataset b table
         * - loads of match table by a direct comparison of dataset tables
         */
        private void AppendByExact(String tableName) throws Exception {

            try {
                        if (!tableName.equalsIgnoreCase(TablesName.nameMatch())) {
                            return;
                        }
                        String[] varA=rsc.getVariableReconciled("A");
                        String[] varB=rsc.getVariableReconciled("B");
                        String dsaColumns = "A.KEY_DSA ";
			String dsbColumns = "B.KEY_DSB ";
			String tdsaColumns = "A_KEY_DSA ";
			String tdsbColumns = "B_KEY_DSB ";
			for (int ind=0;ind<varA.length;ind++) {
			       dsaColumns = dsaColumns + ", A."+varA[ind].toUpperCase();
			       dsbColumns = dsbColumns + ", B."+varB[ind].toUpperCase();
                               tdsaColumns = tdsaColumns + ", A_"+varA[ind].toUpperCase();
			       tdsbColumns = tdsbColumns + ", B_"+varB[ind].toUpperCase();
			}
			
			String matchvariableequality="";
                        String indexcolumn="";
			int numVar=0;
                        String appoMV,mvA, mvB;
			Statement stmMV = (Statement)connection.createStatement();
			ResultSet rsMV = stmMV.executeQuery("SELECT VARIABLE_NAME FROM "+TablesName.nameMatchingVeriables());
			while(rsMV.next()){
                                appoMV = rsMV.getString(1).toUpperCase();
                                mvA=rsc.getAttribute(appoMV, "A");
                                mvB=rsc.getAttribute(appoMV, "B");
				if (numVar> 0) {
                                    matchvariableequality = matchvariableequality + " AND ";
                                    if(numVar < 2)
                                        indexcolumn = indexcolumn + ", ";
                                }   
                                
				matchvariableequality = matchvariableequality+"A."+mvA+"=B."+mvB+" AND A."+mvA+"<>''";
                                if(numVar < 2)
                                    indexcolumn = indexcolumn + mvB;
                                numVar++;
                        }
                        
                        try {
                            Statement stmDrop = (Statement)connection.createStatement();
                            stmDrop.executeUpdate("ALTER TABLE "+TablesName.nameDsb()+" DROP INDEX IX_MATCHING_KEY");
                            stmDrop.close();
                        } catch (Exception e) {  
                        }
                        
                        Statement stmCreate = (Statement)connection.createStatement();
                        stmCreate.executeUpdate("CREATE INDEX IX_MATCHING_KEY ON "+TablesName.nameDsb()+" ("+indexcolumn+")");
                        stmCreate.close();
            
			String insertString = "INSERT INTO "+tableName+" ("+tdsaColumns+","+tdsbColumns+") SELECT "+dsaColumns+","+dsbColumns+
			                      " FROM "+TablesName.nameDsa()+" A, "+TablesName.nameDsb()+" B WHERE "+matchvariableequality;
                                                
                         /* dedup option */
                        VariablesSetting vsdedup = new VariablesSetting(connection);
                        if (vsdedup.ReadProjectType().equalsIgnoreCase("Deduplication")) {
                            insertString = insertString + " AND A.KEY_DSA > B.KEY_DSB";
                        }
                        
                        log.AddMessage("WHERE "+matchvariableequality,false);
			
                 	Statement stmInsert = (Statement)connection.createStatement();
                        stmInsert.executeUpdate(insertString);
                        stmInsert.close();	
		} catch (Exception ex){
		    throw ex;
		}

        }
        
	
        /* creation and load of result table using class methods */
	public void CreateTableResult(String resultType, String pilotVariable, String orderKey) throws Exception {
		String strAndCondition;
		String newTable;
		String tableMU,tableMUBlock;
		String dataTable = TablesName.nameSearchSpaceFinal();
	
		if (resultType.equals("MATCH")) {
			strAndCondition = "AND MU."+pilotVariable+" >= "+this.valThresholdM;
			newTable = TablesName.nameMatch();
		}
		else if (resultType.equals("POSSIBLEMATCH")) {
			strAndCondition = "AND MU."+pilotVariable+" >= "+this.valThresholdU+" AND MU."+pilotVariable+" < "+this.valThresholdM;
			newTable = TablesName.namePossibleMatch();
		}else  {
			strAndCondition = "AND MU."+pilotVariable+" < "+this.valThresholdU;
			newTable = TablesName.nameUnMatch();
		}
	try {
		this.CreateTable(newTable);
                if (model.startsWith("Deterministic"))
                    tableMU = TablesName.nameMURule();
                else
                    tableMU = TablesName.nameMU();

		if (searchMethod.endsWith("Blocking") && !model.startsWith("Deterministic")) {
                        log.AddMessage("Creation of index on block_number","I",false);
                        CreateIndexBlock(dataTable);

			Statement stmBlock = (Statement) connection.createStatement();
			ResultSet rsBlock = stmBlock.executeQuery("SELECT BLOCK_NUMBER FROM "+TablesName.nameBlockModality());
			while (rsBlock.next()) {
				tableMUBlock = tableMU+"_"+rsBlock.getString(1).toUpperCase();
				Statement stm0 =(Statement)connection.createStatement();
            	ResultSet rs0 = stm0.executeQuery("SELECT TABLE_NAME FROM information_schema.TABLES WHERE upper(TABLE_NAME) = '"+tableMUBlock+"' AND TABLE_SCHEMA = 'relais'");
            	if(rs0.next()){
            		this.AppendTable(newTable, tableMUBlock, strAndCondition, dataTable, rsBlock.getString(1), orderKey);
            	}
                else {
                    /* if not exists mu table for the block is used the mu_table created from external marginals */
                    stm0 =(Statement)connection.createStatement();
            	    rs0 = stm0.executeQuery("SELECT TABLE_NAME FROM information_schema.TABLES WHERE upper(TABLE_NAME) = '"+tableMU+"' AND TABLE_SCHEMA = 'relais'");
            	    if(rs0.next()){
            		this.AppendTable(newTable, tableMU, strAndCondition, dataTable, rsBlock.getString(1), orderKey);
            	    }
                }
			}
		}
                else if (model.equals("Deterministic Exact")) {
                    this.AppendByExact(newTable);
                }
		else {
		    this.AppendTable(newTable, tableMU, strAndCondition, dataTable, "NO_BLOCK", orderKey);
		}
	} catch (Exception ex){
	    throw ex;
	}
	}
	
        /* returnd the column name for pilote variable */
        private String PiloteVariable() {
            if (model.startsWith("Deterministic"))
                return ("RULE");
            return ("P_POST");
        }
        
        /* returns other columns in struct of result table */
        private String OtherColumnsAppend() {
            if (model.startsWith("Deterministic"))
                return (" ");
            return (", R, P_POST");
        }
  
        /* returns other columns in struct of result table */
        private String OtherColumnsAppendMU() {
            if (model.startsWith("Deterministic"))
                return (" ");
            return (", MU.R, MU.P_POST");
        }

        /* returns other columns to insert in struct of result table */
        private String OtherColumnsCreate() {
            if (model.startsWith("Deterministic"))
                return (" ");
            return (", R double, P_POST double");
        }

        /* creation and load of match table using generic method 
         * - all pairs with p_post >= match threshold are matches
         */
	public void CreateTableMatch(String orderKey) throws Exception {
		this.CreateTableResult("MATCH",PiloteVariable(), orderKey);
	}
	
        /* creation and load of possible match table using generic method 
         * - all pairs with p_post >= unmatch threshold and p_post < match threshold are possible matches
         */
	public void CreateTablePossiblematch(String orderKey) throws Exception {
		this.CreateTableResult("POSSIBLEMATCH",PiloteVariable(), orderKey);
	}
	
        /* creation and load of unmatch table using generic method 
         * - all pairs with p_post < unmatch threshold are unmatches
         */
	public void CreateTableUnmatch(String orderKey) throws Exception {
		this.CreateTableResult("UNMATCH",PiloteVariable(), orderKey);
	}

        /* creation and load of match table using generic method in case of reduction 1:1
         * - all pairs with p_post >= match threshold are matches
         */
	public void CreateTableMatchOtoO() throws Exception {
		this.CreateTableResultOtoO("MATCH",PiloteVariable());
	}
	
        /* creation and load of possiblematch table using generic method in case of reduction 1:1 
         * - all pairs with p_post >= unmatch threshold and p_post < match threshold are possible matches
         */
	public void CreateTablePossiblematchOtoO() throws Exception {
		this.CreateTableResultOtoO("POSSIBLEMATCH",PiloteVariable());
	}
	
        /* creation and load of unmatch table using generic method in case of reduction 1:1
         * - all pairs with p_post < unmatch threshold are unmatches
         */
	public void CreateTableUnmatchOtoO() throws Exception {
		this.CreateTableResultOtoO("UNMATCH",PiloteVariable());
	}
	
        /* creation and load of residual table with records not in match table */
	private void CreateTableUnmatchDS(String dSName, String tableName, String matchTable, String possMatchTable) throws Exception {
	
	String colName="";
	String colNameResult;
        String colNameResult2;
	String masterTable="";
	
	try {
		if (dSName.equalsIgnoreCase("DSA")) {
			colName="KEY_DSA";
			colNameResult="A_KEY_DSA";
			masterTable=TablesName.nameDsa();
                        colNameResult2="B_KEY_DSB";
		}
		else if (dSName.equalsIgnoreCase("DSB")) {
			colName="KEY_DSB";
			colNameResult="B_KEY_DSB";
			masterTable=TablesName.nameDsb();
                        colNameResult2="A_KEY_DSA";
		}
		else {
			System.out.println("The dataset does not exists! Table "+tableName+" not created.");
			throw new Exception("Dataset does not exists");
		}
			
		String indexName= "INDEX_"+colName;
		try{	
			Statement stmDropInd = (Statement)connection.createStatement();
			stmDropInd.executeUpdate("DROP INDEX "+indexName+" on "+matchTable);
			stmDropInd.close();
			}catch(SQLException e){
			}
			
		String createIndexString = "CREATE INDEX "+indexName+" on "+matchTable+" ("+colNameResult+")";
		Statement stmCreateInd = (Statement)connection.createStatement();
		stmCreateInd.executeUpdate(createIndexString);
		stmCreateInd.close();
		
	
		try{	
			Statement stmDrop = (Statement)connection.createStatement();
			stmDrop.executeUpdate("DROP TABLE "+tableName);
			stmDrop.close();
			}catch(SQLException e){
			}
		
		String createString = "CREATE TABLE "+tableName+" as SELECT * from "+masterTable+" ds where not exists "+
		                      " (SELECT 1 from "+matchTable+" m where ds."+colName+"=m."+colNameResult+")";
		
                if (ptype.equalsIgnoreCase("deduplication")) {
                    createString = createString + " and not exists (SELECT 1 from "
                            +matchTable+" m where ds."+colName+"=m."+colNameResult2+")";
                }
                
		Statement stmCreate = (Statement)connection.createStatement();
	    stmCreate.executeUpdate(createString);
	    stmCreate.close();
	} catch (Exception ex){
	    throw ex;
	}
	}

	/* creation and load of residual table for dataset "a" using generic method */
	public void CreateTableUnmatchDSA() throws Exception {
		this.CreateTableUnmatchDS("DSA", TablesName.nameResidualDsa(),TablesName.nameMatch(),TablesName.namePossibleMatch());
	}
	
        /* creation and load of residual table for dataset "b" using generic method */
	public void CreateTableUnmatchDSB() throws Exception {
		this.CreateTableUnmatchDS("DSB", TablesName.nameResidualDsb(),TablesName.nameMatch(),TablesName.namePossibleMatch());
	}
	
        /* loads record in result table in case of reduction 1:1 */
	private void AppendTableOtoO(String tableName, String AndCondition, String dataTableName) throws Exception {
		try {
                        String[] varA=rsc.getVariableReconciled("A");
                        String[] varB=rsc.getVariableReconciled("B");
                        String dsaColumns = "A.KEY_DSA ";
			String dsbColumns = "B.KEY_DSB ";
			String tdsaColumns = "A_KEY_DSA ";
			String tdsbColumns = "B_KEY_DSB ";
			for (int ind=0;ind<varA.length;ind++) {
			       dsaColumns = dsaColumns + ", A."+varA[ind].toUpperCase();
			       dsbColumns = dsbColumns + ", B."+varB[ind].toUpperCase();
                               tdsaColumns = tdsaColumns + ", A_"+varA[ind].toUpperCase();
			       tdsbColumns = tdsbColumns + ", B_"+varB[ind].toUpperCase();
			}
			
			String insertString = "INSERT INTO "+tableName+" ("+tdsaColumns+","+tdsbColumns+OtherColumnsAppend()+") SELECT "+dsaColumns+","+dsbColumns+OtherColumnsAppendMU()+
			                      " FROM "+TablesName.nameDsa()+" A, "+TablesName.nameDsb()+" B, "+dataTableName+" MU WHERE A.KEY_DSA=MU.i AND B.KEY_DSB=MU.j "+AndCondition;
			
			Statement stmCreate = (Statement)connection.createStatement();
		    stmCreate.executeUpdate(insertString);
		    stmCreate.close();	
		} catch (Exception ex){
		    throw ex;
		}
		}

        /* creation and load of result table using class methods in case of result 1:1 */
	public void CreateTableResultOtoO(String resultType, String pilotVariable) throws Exception {
		String strAndCondition;
		String newTable;
		String dataTable,dataTableBlock;
	
		if (resultType.equals("MATCH")) {
			strAndCondition = "AND MU."+pilotVariable+" >= "+this.valThresholdM;
			newTable = TablesName.nameMatch();
		}
		else if (resultType.equals("POSSIBLEMATCH")) {
			strAndCondition = "AND MU."+pilotVariable+" >= "+this.valThresholdU+" AND MU."+pilotVariable+" < "+this.valThresholdM;
			newTable = TablesName.namePossibleMatch();
		}else  {
			strAndCondition = "AND MU."+pilotVariable+" < "+this.valThresholdU;
			newTable = TablesName.nameUnMatch();
		}
	try {
		this.CreateTable(newTable);
		dataTable = TablesName.nameOtoO();
		if (searchMethod.endsWith("Blocking")) {
			Statement stmBlock = (Statement) connection.createStatement();
			ResultSet rsBlock = stmBlock.executeQuery("SELECT BLOCK_NUMBER FROM "+TablesName.nameBlockModality());
			while (rsBlock.next()) {
        		dataTableBlock = dataTable+"_"+this.blockVar.replace(",","_").toUpperCase()+"_"+rsBlock.getString(1).toUpperCase();
			Statement stm0 =(Statement)connection.createStatement();
            	ResultSet rs0 = stm0.executeQuery("SELECT TABLE_NAME FROM information_schema.TABLES WHERE UPPER(TABLE_NAME) = '"+dataTableBlock.toUpperCase()+"' AND UPPER(TABLE_SCHEMA) = 'RELAIS'");
            	if(rs0.next()){
            		this.AppendTableOtoO(newTable, strAndCondition, dataTableBlock);
            	}
            	rs0.close();
            	stm0.close();
			}
		}
		else {
			this.AppendTableOtoO(newTable, strAndCondition, dataTable);
		}

	} catch (Exception ex){
	    throw ex;
	}
	}
}
