package it.istat.relais.searchSpaceCreation;

import it.istat.relais.constants.TablesName;
import it.istat.relais.project.ReconciledSchema;
import it.istat.relais.project.VariablesSetting;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.StringTokenizer;


public class Blocking {
	Connection connection;
	private String method;
	private ArrayList<String> bm;
	private String blockingKey;
        private ReconciledSchema rsc;
	
	public Blocking(Connection conn){
		this.connection = conn;
		this.method= "Blocking";
		this.bm = new ArrayList<String>();
                rsc = new ReconciledSchema(conn);
	}
	
        /* creation of reduced search space table by blocking method
         * operations of the module:
         * - creation of table named using blocking keys and block number
         * - creation of temporary table for dataset "b" with the blockvalue of record and an index
         * - loads records in the table by comparing dataset "a" and temporary table
         */
	public String CreateBlocks(String blockingKey) throws Exception {
        Statement stmDrop, stmCreate, stmInsert;
        String tableNameBlock = TablesName.nameSearchSpaceTemp();
        String concatVarDsa, concatVarDsb;
                
        this.blockingKey = blockingKey;
        try{
            StringTokenizer tok = new StringTokenizer(blockingKey,",");
            String comVar=tok.nextToken();
            concatVarDsa = "CONCAT("+rsc.getAttribute(comVar, "A");
            concatVarDsb = "CONCAT("+rsc.getAttribute(comVar, "B");
            while (tok.hasMoreTokens()) {
                comVar=tok.nextToken();
                concatVarDsa = concatVarDsa+",'_',"+rsc.getAttribute(comVar, "A");
                concatVarDsb = concatVarDsb+",'_',"+rsc.getAttribute(comVar, "B");
            }
            concatVarDsa = concatVarDsa+")";
            concatVarDsb = concatVarDsb+")";
                        
            Statement stmDropIniAppo =(Statement)connection.createStatement();
            stmDropIniAppo.executeUpdate("DROP TABLE IF EXISTS DSBAPPO");
            stmDropIniAppo.close();
            Statement stmCreateAppo = (Statement)connection.createStatement();
            stmCreateAppo.executeUpdate("CREATE TABLE DSBAPPO (KEY_DSB integer unsigned, BLOCK varchar(500))");
            stmCreateAppo.close();
            Statement stmLoadAppo = (Statement)connection.createStatement();
            stmLoadAppo.executeUpdate("INSERT INTO DSBAPPO (KEY_DSB,BLOCK) SELECT  KEY_DSB, ifnull("+concatVarDsb+",' ')  FROM "+TablesName.nameDsb());
            stmLoadAppo.close();

            Statement stmCreateIx = (Statement)connection.createStatement();
            stmCreateIx.executeUpdate("CREATE INDEX IX_VAR_BLOCK ON DSBAPPO (BLOCK)");
            stmCreateIx.close();
                        
            try{	
            stmDrop = (Statement)connection.createStatement();
            stmDrop.executeUpdate("DROP table "+tableNameBlock);
            stmDrop.close();
            }catch(SQLException e){
            }
            stmCreate = (Statement)connection.createStatement();
            String createString = "CREATE TABLE "+tableNameBlock+" (";
            createString = createString+" KEY_DSA INTEGER UNSIGNED, KEY_DSB INTEGER UNSIGNED,";
            createString = createString +" BLOCK VARCHAR(500), ROWNUMBER BIGINT UNSIGNED NOT NULL AUTO_INCREMENT, PRIMARY KEY (ROWNUMBER)) ENGINE = MYISAM;";

	    stmCreate.executeUpdate(createString);
	    stmCreate.close();
	    
	    String insertString = "insert into "+tableNameBlock+" (KEY_DSA, KEY_DSB, BLOCK)";
	    insertString = insertString + " select a.KEY_DSA, b.KEY_DSB, b.BLOCK";
	    insertString = insertString + " from "+TablesName.nameDsa()+" a, DSBAPPO b";
	    insertString = insertString + " where ifnull("+concatVarDsa+",' ')=b.BLOCK";
	     /* dedup option */
            VariablesSetting vsdedup = new VariablesSetting(connection);
            if (vsdedup.ReadProjectType().equalsIgnoreCase("Deduplication")) {
                insertString = insertString + " and A.KEY_DSA > B.KEY_DSB";
            }
            
	    stmInsert = (Statement)connection.createStatement();
	    stmInsert.executeUpdate(insertString);
	    stmInsert.close();
	    
            Statement stmDropAppo =(Statement)connection.createStatement();
            stmDropAppo.executeUpdate("DROP TABLE DSBAPPO");
            stmDropAppo.close();
            
            try {
                Statement stmDropBM = (Statement)connection.createStatement();
                stmDropBM.executeUpdate("DROP table "+TablesName.nameBlockModality());
            } catch(SQLException e) {
            }	
            Statement stmCreateBM =(Statement)connection.createStatement();
            stmCreateBM.executeUpdate("Create table "+TablesName.nameBlockModality()+" (BLOCK_NUMBER INTEGER UNSIGNED NOT NULL AUTO_INCREMENT, BLOCK_MODALITY VARCHAR(500), FREQUENCY BIGINT ,PRIMARY KEY (BLOCK_NUMBER)) ENGINE = MYISAM;");
            stmCreateBM.close();

	    String insCBMString = "insert into "+TablesName.nameBlockModality()+"(BLOCK_MODALITY,FREQUENCY) SELECT  BLOCK, count(*) FROM "+tableNameBlock+" group by BLOCK;";
	    
	    Statement stmBM =(Statement)connection.createStatement();
	    stmBM.executeUpdate(insCBMString);
            stmBM.close();
        }catch (Exception e) {
            e.printStackTrace();		
            throw e;
	}
		return this.method;
	}
	
	
			
}