package it.istat.relais.searchSpaceCreation;

import it.istat.relais.constants.TablesName;
import it.istat.relais.project.ReconciledSchema;
import it.istat.relais.project.VariablesSetting;
import java.sql.Connection;
import java.sql.Statement;
import java.util.StringTokenizer;


public class NestedBlocking {
	
	Connection connection;
	private String method;
        private ReconciledSchema rsc;
	
	public NestedBlocking(Connection conn){
		this.connection=conn;
		this.method = "Nested Blocking";
                rsc = new ReconciledSchema(conn);
	}

        /* creation of reduced search space table by combined blocking method
         * operations of the module:
         * - creation of the output table
         * - creation of a temporary table with records from both datasets 
         * - creation of a second temporary table with records sorted by blocking and sorting keys
         * - loads record in the table by comparing pairs of records in the second temporary table
         */
	public String CreateSSR(String blockingKey, String sortingKey, int windowSize) throws Exception {
            Statement stmDrop, stmCreate, stmInsert, stmDrop1, stmCreate1, stmInsert1, stmDrop2, stmCreate2, stmInsert2;
            String tableNameBlock = TablesName.nameSearchSpaceTemp();
            String concatVarDsa, concatVarDsb;
            String sortVarDsa, sortVarDsb;
		
            VariablesSetting vs = new VariablesSetting(connection);
            vs.UpdateWindowSize(windowSize);
            try {
                StringTokenizer tok = new StringTokenizer(blockingKey,",");
                String comVar=tok.nextToken();
                concatVarDsa = "CONCAT("+rsc.getAttribute(comVar, "A");
                concatVarDsb = "CONCAT("+rsc.getAttribute(comVar, "B");
                while (tok.hasMoreTokens()) {
                    comVar=tok.nextToken();
                    concatVarDsa = concatVarDsa+",'_',"+rsc.getAttribute(comVar, "A");
                    concatVarDsb = concatVarDsb+",'_',"+rsc.getAttribute(comVar, "B");
                }
                concatVarDsa = concatVarDsa+")";
                concatVarDsb = concatVarDsb+")";
                
                tok = new StringTokenizer(sortingKey,",");
                comVar=tok.nextToken();
                sortVarDsa = rsc.getAttribute(comVar, "A");
                sortVarDsb = rsc.getAttribute(comVar, "B");
                while (tok.hasMoreTokens()) {
                    comVar=tok.nextToken();
                    sortVarDsa = sortVarDsa+",' ',"+rsc.getAttribute(comVar, "A");
                    sortVarDsb = sortVarDsb+",' ',"+rsc.getAttribute(comVar, "B");
                }
                
		stmDrop =(Statement)connection.createStatement();
		stmDrop.executeUpdate("DROP table IF EXISTS "+tableNameBlock+";");
		stmDrop.close();

		stmCreate=(Statement)connection.createStatement();
		String createString ="CREATE TABLE "+tableNameBlock+" (KEY_DSA INTEGER UNSIGNED, KEY_DSB INTEGER UNSIGNED, BLOCK VARCHAR(500),"+
                                     " ROWNUMBER BIGINT UNSIGNED NOT NULL AUTO_INCREMENT, PRIMARY KEY (ROWNUMBER)) ENGINE = MYISAM;";
		stmCreate.executeUpdate(createString);
                stmCreate.close();

                stmDrop1 = (Statement)connection.createStatement();
                stmDrop1.executeUpdate("DROP table IF EXISTS TEMP1_SNM;");
                stmDrop1.close();
		
                stmCreate1 =(Statement)connection.createStatement();
                String create1String = "CREATE TABLE TEMP1_SNM (DS_TYPE CHAR(1),";
                create1String = create1String+ " KEY_DS INTEGER UNSIGNED NOT NULL, BLOCK varchar(500),";
                create1String = create1String+ " SORTING_KEY VARCHAR(500)) ENGINE=MYISAM;";
                stmCreate1.executeUpdate(create1String);
                stmCreate1.close();

                stmInsert1=(Statement)connection.createStatement();
                String insert1String = "insert into TEMP1_SNM (DS_TYPE, KEY_DS, BLOCK, SORTING_KEY)"+
                                       " (select 'A', KEY_DSA, "+concatVarDsa+
                                       ", concat(" +sortVarDsa+ ") from "+TablesName.nameDsa()+");";
                stmInsert1.executeUpdate(insert1String);

                String insert2String = "insert into TEMP1_SNM (DS_TYPE, KEY_DS, BLOCK, SORTING_KEY)"+
                                       " (select 'B', KEY_DSB, "+concatVarDsb+
                                       ", concat(" +sortVarDsb+ ") from "+TablesName.nameDsb()+");";
                stmInsert1.executeUpdate(insert2String);
                stmInsert1.close();

                Statement stmCreateIx = (Statement)connection.createStatement();
                stmCreateIx.executeUpdate("CREATE INDEX IX_VAR_BLOCK ON TEMP1_SNM (BLOCK)");
                stmCreateIx.close();

                stmDrop2=(Statement)connection.createStatement();
                stmDrop2.executeUpdate("DROP table IF EXISTS TEMP2_SNM;");
                stmDrop2.close();

	    stmCreate2=(Statement)connection.createStatement();
	    String create2String = "CREATE TABLE TEMP2_SNM (KEY_SNM INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,";
	    create2String = create2String + " DS_TYPE CHAR(1), KEY_DS INTEGER UNSIGNED NOT NULL, BLOCK VARCHAR(500), ";
	    create2String = create2String + "SORTING_KEY VARCHAR(500), PRIMARY KEY (KEY_SNM)) ENGINE=MYISAM;";
        stmCreate2.executeUpdate(create2String);
        stmCreate2.close();
	
        stmInsert2=(Statement)connection.createStatement();
        String insert3String = "insert into TEMP2_SNM (DS_TYPE, KEY_DS, BLOCK, ";
        insert3String = insert3String+"SORTING_KEY) select DS_TYPE, KEY_DS, BLOCK, ";
        insert3String = insert3String+"SORTING_KEY from TEMP1_SNM order by BLOCK, SORTING_KEY;";
        stmInsert2.executeUpdate(insert3String);
        stmInsert2.close();

        stmDrop1 = (Statement)connection.createStatement();
        stmDrop1.executeUpdate("DROP table TEMP1_SNM;");
        stmDrop1.close();

        VariablesSetting vsdedup = new VariablesSetting(connection);
        int windowSizeDB = (windowSize-1);
        if (vsdedup.ReadProjectType().equalsIgnoreCase("Deduplication")) {
            windowSizeDB = windowSizeDB*2;
        }
        stmInsert = (Statement)connection.createStatement();   
        String insertString = "insert into "+tableNameBlock+" (KEY_DSA, KEY_DSB, BLOCK) select a.KEY_DS, b.KEY_DS, a.BLOCK ";
        insertString = insertString + "	from TEMP2_SNM a, TEMP2_SNM b where a.BLOCK=b.BLOCK and a.DS_TYPE = 'A' and b.DS_TYPE = 'B' ";
        insertString = insertString + " and (cast(b.KEY_SNM as signed) - cast(a.KEY_SNM as signed)) between -"+ windowSize +" and "+ windowSize;
         /* dedup option */
        if (vsdedup.ReadProjectType().equalsIgnoreCase("Deduplication")) {
            insertString = insertString + " and A.KEY_SNM > B.KEY_SNM";
        }
        
        stmInsert.executeUpdate(insertString);
        stmInsert.close();
	
        stmDrop2=(Statement)connection.createStatement();
        stmDrop2.executeUpdate("DROP table TEMP2_SNM;");
        stmDrop2.close();
        

        Statement stmDropBM = (Statement)connection.createStatement();
        stmDropBM.executeUpdate("DROP table IF EXISTS "+TablesName.nameBlockModality());
            
        Statement stmCreateBM =(Statement)connection.createStatement();
        stmCreateBM.executeUpdate("Create table "+TablesName.nameBlockModality()+" (BLOCK_NUMBER INTEGER UNSIGNED NOT NULL AUTO_INCREMENT, BLOCK_MODALITY VARCHAR(500), FREQUENCY BIGINT, PRIMARY KEY (BLOCK_NUMBER)) ENGINE = MYISAM;");
        stmCreateBM.close();

        String insCBMString = "insert into "+TablesName.nameBlockModality()+"(BLOCK_MODALITY,FREQUENCY) SELECT  BLOCK, count(*) FROM "+tableNameBlock+" group by BLOCK;";

        Statement stmBM =(Statement)connection.createStatement();
        stmBM.executeUpdate(insCBMString);
        stmBM.close();

        }catch (Exception e) {
			e.printStackTrace();		
                        throw e;
	}
	    return this.method;

	}
}
