package it.istat.relais.searchSpaceCreation;

import it.istat.relais.constants.TablesName;
import it.istat.relais.project.ReconciledSchema;
import it.istat.relais.project.VariablesSetting;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.StringTokenizer;


public class SortedNeighborhood {
	
	Connection connection;
	private String method;
        private ReconciledSchema rsc;
	
	public SortedNeighborhood(Connection conn){
		this.connection=conn;
		this.method = "SNM";
                rsc = new ReconciledSchema(conn);
	}

        /* creation of reduced search space table by sorted neighborhood method
         * operations of the module:
         * - creation of the output table
         * - creation of a temporary table with records from both datasets 
         * - creation of a second temporary table with records sorted by sorting keys
         * - loads record in the table by comparing pairs of records in the second temporary table
         */
	public String CreateSNM(String blockingKey, int windowSize) throws Exception {
		
            Statement stmDrop, stmCreate, stmInsert, stmDrop1, stmCreate1, stmInsert1, stmDrop2, stmCreate2, stmInsert2;
            String tableNameSnm = TablesName.nameSearchSpaceTemp();
            String concatVarDsa, concatVarDsb;
		
            VariablesSetting vs = new VariablesSetting(connection);
            vs.UpdateWindowSize(windowSize);
	try{
            StringTokenizer tok = new StringTokenizer(blockingKey,",");
            String comVar=tok.nextToken();
            concatVarDsa = rsc.getAttribute(comVar, "A");
            concatVarDsb = rsc.getAttribute(comVar, "B");
            while (tok.hasMoreTokens()) {
                comVar=tok.nextToken();
                concatVarDsa = concatVarDsa+",' ',"+rsc.getAttribute(comVar, "A");
                concatVarDsb = concatVarDsb+",' ',"+rsc.getAttribute(comVar, "B");
            }

		stmDrop =(Statement)connection.createStatement();
		stmDrop.executeUpdate("DROP table IF EXISTS "+tableNameSnm+";");
		stmDrop.close();

		stmCreate=(Statement)connection.createStatement();
		String createString ="CREATE TABLE "+tableNameSnm+" (KEY_DSA INTEGER UNSIGNED, KEY_DSB INTEGER UNSIGNED, ROWNUMBER BIGINT UNSIGNED NOT NULL AUTO_INCREMENT, PRIMARY KEY (ROWNUMBER)) ENGINE = MYISAM;";
		stmCreate.executeUpdate(createString);
        stmCreate.close();
        
        stmDrop1 = (Statement)connection.createStatement();
	    stmDrop1.executeUpdate("DROP table IF EXISTS TEMP1_SNM;");
	    stmDrop1.close();
		
	    stmCreate1 =(Statement)connection.createStatement();
	    String create1String = "CREATE TABLE TEMP1_SNM (DS_TYPE CHAR(1),"+
	    	                   " KEY_DS INTEGER UNSIGNED NOT NULL,"+
	    	                   " SORTING_KEY VARCHAR(500)) ENGINE=MYISAM;";
        stmCreate1.executeUpdate(create1String);
        stmCreate1.close();
        
        stmInsert1=(Statement)connection.createStatement();
        String insert1String = "insert into TEMP1_SNM (DS_TYPE, KEY_DS, ";
        insert1String = insert1String+ "SORTING_KEY)(select 'A', KEY_DSA, concat(";
        insert1String = insert1String+ concatVarDsa+") from "+TablesName.nameDsa()+") union all (";
        insert1String = insert1String+ "select 'B', KEY_DSB, concat("+concatVarDsb+") from "+TablesName.nameDsb()+");";
        stmInsert1.executeUpdate(insert1String);
        stmInsert1.close();
        
        stmDrop2=(Statement)connection.createStatement();
	    stmDrop2.executeUpdate("DROP table IF EXISTS TEMP2_SNM;");
	    stmDrop2.close();

	    stmCreate2=(Statement)connection.createStatement();
	    String create2String = "CREATE TABLE TEMP2_SNM (KEY_SNM INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,"+
	    	                   " DS_TYPE CHAR(1), KEY_DS INTEGER UNSIGNED NOT NULL, "+
	    	                   "SORTING_KEY VARCHAR(500), PRIMARY KEY (KEY_SNM)) ENGINE=MYISAM;";
        stmCreate2.executeUpdate(create2String);
        stmCreate2.close();
        	
        stmInsert2=(Statement)connection.createStatement();
        String insert2String = "insert into TEMP2_SNM (DS_TYPE, KEY_DS, "+
                                "SORTING_KEY) select DS_TYPE, KEY_DS, "+
                                "SORTING_KEY from TEMP1_SNM order by SORTING_KEY,KEY_DS,DS_TYPE;";
        stmInsert2.executeUpdate(insert2String);
        stmInsert2.close();
        
        try{
        stmDrop1 = (Statement)connection.createStatement();
	    stmDrop1.executeUpdate("DROP table TEMP1_SNM;");
	    stmDrop1.close();
        }catch(SQLException e){
        	;
        }
        VariablesSetting vsdedup = new VariablesSetting(connection);
        int windowSizeDB = (windowSize-1);
        if (vsdedup.ReadProjectType().equalsIgnoreCase("Deduplication")) {
            windowSizeDB = windowSizeDB*2;
        }
        stmInsert = (Statement)connection.createStatement();   
        String insertString = "insert into "+tableNameSnm+" (KEY_DSA, KEY_DSB) select a.KEY_DS, b.KEY_DS ";
        insertString = insertString + "	from TEMP2_SNM a, TEMP2_SNM b where a.DS_TYPE = 'A' and b.DS_TYPE = 'B' ";
        insertString = insertString + " and (cast(b.KEY_SNM as signed) - cast(a.KEY_SNM as signed)) between -"+ windowSizeDB +" and "+ windowSizeDB;
         /* dedup option */
        
        if (vsdedup.ReadProjectType().equalsIgnoreCase("Deduplication")) {
            insertString = insertString + " and A.KEY_SNM > B.KEY_SNM";
        }
        
        stmInsert.executeUpdate(insertString);
        stmInsert.close();
		
        stmDrop2=(Statement)connection.createStatement();
	    stmDrop2.executeUpdate("DROP table TEMP2_SNM;");
	    stmDrop2.close();
            
        }catch (Exception e) {
			e.printStackTrace();		
                        throw e;
	}
	    return this.method;

	}
}
